/*
 * Copyright (c) 2013, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the LICENSE file that accompanied this code.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */

package hello;

import javafx.scene.layout.Region;
import javafx.scene.paint.Color;
import javafx.scene.shape.SVGPath;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathFactory;
import java.io.File;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * The classic SVG Tiger using SVGPath. A little more work and I could make it just use Path, but I'm not
 * sure if there is much benefit to it.
 */
public class Tiger extends Region {
    public Tiger() {
        SVGPath pg6 = new SVGPath();
        pg6.setStrokeWidth(0.17200001);
        pg6.setStroke(Color.web("#000"));
        pg6.setFill(Color.WHITE);
        pg6.setContent("m-122.3,84.285s0.1,1.894-0.73,1.875c-0.82-0.019-17.27-48.094-37.8-45.851,0,0,17.78-7.353,38.53,43.976z");
        getChildren().add(pg6);
        SVGPath pg10 = new SVGPath();
        pg10.setStrokeWidth(0.17200001);
        pg10.setStroke(Color.web("#000"));
        pg10.setFill(Color.WHITE);
        pg10.setContent("m-118.77,81.262s-0.55,1.816-1.32,1.517c-0.77-0.298,0.11-51.104-19.95-55.978,0,0,19.22-0.864,21.27,54.461z");
        getChildren().add(pg10);
        SVGPath pg14 = new SVGPath();
        pg14.setStrokeWidth(0.17200001);
        pg14.setStroke(Color.web("#000"));
        pg14.setFill(Color.WHITE);
        pg14.setContent("m-91.284,123.59s1.636,0.96,1.166,1.64c-0.471,0.67-49.642-12.13-59.102,6.23,0,0,3.68-18.89,57.936-7.87z");
        getChildren().add(pg14);
        SVGPath pg18 = new SVGPath();
        pg18.setStrokeWidth(0.17200001);
        pg18.setStroke(Color.web("#000"));
        pg18.setFill(Color.WHITE);
        pg18.setContent("m-94.093,133.8s1.856,0.4,1.622,1.19c-0.233,0.79-50.939,4.13-54.129,24.53,0,0-2.46-19.08,52.507-25.72z");
        getChildren().add(pg18);
        SVGPath pg22 = new SVGPath();
        pg22.setStrokeWidth(0.17200001);
        pg22.setStroke(Color.web("#000"));
        pg22.setFill(Color.WHITE);
        pg22.setContent("m-98.304,128.28s1.778,0.66,1.432,1.41-50.998-3.34-57.128,16.37c0,0,0.35-19.24,55.696-17.78z");
        getChildren().add(pg22);
        SVGPath pg26 = new SVGPath();
        pg26.setStrokeWidth(0.17200001);
        pg26.setStroke(Color.web("#000"));
        pg26.setFill(Color.WHITE);
        pg26.setContent("m-109.01,110.07s1.31,1.38,0.67,1.9-44.38-25.336-58.53-10.29c0,0,8.74-17.147,57.86,8.39z");
        getChildren().add(pg26);
        SVGPath pg30 = new SVGPath();
        pg30.setStrokeWidth(0.17200001);
        pg30.setStroke(Color.web("#000"));
        pg30.setFill(Color.WHITE);
        pg30.setContent("m-116.55,114.26s1.45,1.22,0.88,1.81c-0.58,0.59-46.97-20.148-59.32-3.6,0,0,6.74-18.023,58.44,1.79z");
        getChildren().add(pg30);
        SVGPath pg34 = new SVGPath();
        pg34.setStrokeWidth(0.17200001);
        pg34.setStroke(Color.web("#000"));
        pg34.setFill(Color.WHITE);
        pg34.setContent("m-119.15,118.34s1.6,1,1.11,1.67c-0.49,0.66-49.27-13.56-59.25,4.51,0,0,4.22-18.77,58.14-6.18z");
        getChildren().add(pg34);
        SVGPath pg38 = new SVGPath();
        pg38.setStrokeWidth(0.17200001);
        pg38.setStroke(Color.web("#000"));
        pg38.setFill(Color.WHITE);
        pg38.setContent("m-108.42,118.95s1.12,1.53,0.42,1.97c-0.7,0.43-40.77-30.818-56.73-17.71,0,0,10.87-15.884,56.31,15.74z");
        getChildren().add(pg38);
        SVGPath pg42 = new SVGPath();
        pg42.setStrokeWidth(0.17200001);
        pg42.setStroke(Color.web("#000"));
        pg42.setFill(Color.WHITE);
        pg42.setContent("m-128.2,90s0.6,1.8-0.2,2-29.4-41.8-48.6-34.2c0,0,15.2-11.8,48.8,32.2z");
        getChildren().add(pg42);
        SVGPath pg46 = new SVGPath();
        pg46.setStrokeWidth(0.17200001);
        pg46.setStroke(Color.web("#000"));
        pg46.setFill(Color.WHITE);
        pg46.setContent("m-127.5,96.979s0.97,1.629,0.23,1.996c-0.74,0.368-37.72-34.476-54.83-22.914,0,0,12.3-14.8,54.6,20.918z");
        getChildren().add(pg46);
        SVGPath pg50 = new SVGPath();
        pg50.setStrokeWidth(0.17200001);
        pg50.setStroke(Color.web("#000"));
        pg50.setFill(Color.WHITE);
        pg50.setContent("m-127.62,101.35s1.12,1.53,0.42,1.97c-0.7,0.43-40.77-30.818-56.73-17.713,0,0,10.87-15.881,56.31,15.743z");
        getChildren().add(pg50);
        SVGPath pg54 = new SVGPath();
        pg54.setStroke(Color.web("#000"));
        pg54.setFill(Color.WHITE);
        pg54.setContent("m-129.83,103.06c0.5,6.05,1.49,12.62,3.23,15.74,0,0-3.6,12.4,5.2,25.6,0,0-0.4,7.2,1.2,10.4,0,0,4,8.4,8.8,9.2,3.88,0.65,12.607,3.72,22.468,5.12,0,0,17.132,14.08,13.932,26.88,0,0-0.4,16.4-4,18,0,0,11.6-11.2,2,5.6l-4.4,18.8s25.6-21.6,10-3.2l-10,26s19.6-18.4,12.4-10l-3.2,8.8s43.2-27.2,12.4,2.4c0,0,8-3.6,12.4-0.8,0,0,6.8-1.2,6,0.4,0,0-20.8,10.4-24.4,28.8,0,0,8.4-10,5.2,0.8l0.4,11.6s4-21.6,3.6,16c0,0,19.2-18,7.6,2.8v16.8s15.2-16.4,8.8-3.6c0,0,10-8.8,6,6.4,0,0-0.8,10.4,3.6-0.8,0,0,16-30.6,10-4.4,0,0-0.8,19.2,4,4.4,0,0,0.4,10.4,9.6,17.6,0,0-1.2-50.8,11.6-14.8l4,16.4s2.8-9.2,2.4-14.4l8,8s15.2-22.8,12-9.6c0,0-7.6,16-6,20.8,0,0,16.8-34.8,18-36.4,0,0-2,42.4,8.8,6.4,0,0,5.6,12,2.8,16.4,0,0,8-8,7.2-11.2,0,0,4.6-8.2,7.4,5.4,0,0,1.8,9.4,3.4,6.2,0,0,4,24,5.2,1.2,0,0,1.6-13.6-5.6-25.2,0,0,0.8-3.2-2-7.2,0,0,13.6,21.6,6.4-7.2,0,0,11.201,8,12.401,8,0,0-13.601-23.2-4.801-18.4,0,0-5.2-10.4,12.801,1.6,0,0-16.001-16,1.6-6.4,0,0,7.999,6.4,0.4-3.6,0,0-14.401-16,7.599,2,0,0,11.6,16.4,12.4,19.2,0,0-10-29.2-14.4-32,0,0,8.4-36.4,49.6-20.8,0,0,6.8,17.2,11.2-1.2,0,0,12.8-6.4,24,21.2,0,0,4-13.6,3.2-16.4,0,0,6.8,1.2,6,0,0,0,13.2,4.4,14.4,3.6,0,0,6.8,6.8,7.2,3.2,0,0,9.2,2.8,7.2-0.8,0,0,8.8,15.6,9.2,19.2l2.4-14,2,2.8s1.6-7.6,0.8-8.8,20,6.8,24.8,27.6l2,8.4s6-14.8,4.4-18.8c0,0,5.2,0.8,5.6,5.2,0,0,4-23.2-0.8-29.2,0,0,4.4-0.8,5.6,2.8v-7.2s7.2,0.8,7.2-1.6c0,0,4.4-4,6.4,0.8,0,0-12.4-35.2,6-16,0,0,7.2,10.8,3.6-8s-7.6-20.4-2.8-20.8c0,0,0.8-3.6-1.2-5.2s1.2,0,1.2,0,4.8,4-0.4-18c0,0,6.4,1.6-5.6-27.6,0,0,2.8-2.4-1.2-10.8,0,0,8,4.4,10.8,2.8,0,0-0.4-1.6-3.6-5.6,0,0-21.6-54.8-1.2-32.8,0,0,11.85,13.55,5.45-9.25,0,0-9.11-24.009-8.33-28.305l-429.55,23.015z");
        getChildren().add(pg54);
        SVGPath pg58 = new SVGPath();
        pg58.setStroke(Color.web("#000"));
        pg58.setFill(Color.web("#cc7226"));
        pg58.setContent("m299.72,80.245c0.62,0.181,2.83,1.305,4.08,2.955,0,0,6.8,10.8,1.6-7.6,0,0-9.2-28.8-0.4-17.6,0,0,6,7.2,2.8-6.4-3.86-16.427-6.4-22.8-6.4-22.8s11.6,4.8-15.2-34.8l8.8,3.6s-19.6-39.6-41.2-44.8l-8-6s38.4-38,25.6-74.8c0,0-6.8-5.2-16.4,4,0,0-6.4,4.8-12.4,3.2,0,0-30.8,1.2-32.8,1.2s-36.8-37.2-102.4-19.6c0,0-5.2,2-9.599,0.8,0,0-18.401-16-67.201,6.8,0,0-10,2-11.6,2s-4.4,0-12.4,6.4-8.4,7.2-10.4,8.8c0,0-16.4,11.2-21.2,12,0,0-11.6,6.4-16,16.4l-3.6,1.2s-1.6,7.2-2,8.4c0,0-4.8,3.6-5.6,9.2,0,0-8.8,6-8.4,10.4,0,0-1.6,5.2-2.4,10,0,0-7.2,4.8-6.4,7.6,0,0-7.6,14-6.4,20.8,0,0-6.4-0.4-9.2,2,0,0-0.8,4.8-2.4,5.2,0,0-2.8,1.2-0.4,5.2,0,0-1.6,2.8-2,4.4,0,0,0.8,2.8-3.6,8.4,0,0-6.4,18.8-4.4,24,0,0,0.4,4.8-2.4,6.4,0,0-3.6-0.4,4.8,11.6,0,0,0.8,1.2-2.4,3.6,0,0-17.2,3.6-19.6,20,0,0-13.6,14.8-13.6,20,0,2.305,0.27,5.452,0.97,10.06,0,0-0.57,8.34,27.03,9.14s402.72-31.355,402.72-31.355z");
        getChildren().add(pg58);
        SVGPath pg62 = new SVGPath();
        pg62.setFill(Color.web("#cc7226"));
        pg62.setContent("m-115.6,102.6c-25-39.4-10.6,17-10.6,17,8.8,34.4,138.4-3.2,138.4-3.2s168.8-30.4,180-34.4,106.4,2.4,106.4,2.4l-5.6-16.8c-64.8-46.4-84-23.2-97.6-27.2s-11.2,5.6-14.4,6.4-42.4-24-48.8-23.2-31.74-22.951-16.8,8.8c16,34-58.4,39.2-75.2,28s7.2,18.4,7.2,18.4c18.4,20-16,3.2-16,3.2-34.4-12.8-58.4,12.8-61.6,13.6s-8,4-8.8-2.4-8.31-23.101-40,3.2c-20,16.6-33.8-5.4-33.8-5.4l-2.8,11.6z");
        getChildren().add(pg62);
        SVGPath pg66 = new SVGPath();
        pg66.setFill(Color.web("#e87f3a"));
        pg66.setContent("m133.51,25.346c-6.4,0.8-31.77-22.939-16.8,8.8,16.6,35.2-58.4,39.2-75.2,28-16.801-11.2,7.2,18.4,7.2,18.4,18.4,20.004-16.001,3.2-16.001,3.2-34.4-12.8-58.4,12.8-61.6,13.6s-8,4.004-8.8-2.4c-0.8-6.4-8.179-22.934-40,3.2-21.236,17.344-34.729-4.109-34.729-4.109l-3.2,10.113c-25-39.804-9.93,18.51-9.93,18.51,8.81,34.4,139.06-4.51,139.06-4.51s168.8-30.404,180-34.404,105.53,2.327,105.53,2.327l-5.53-17.309c-64.8-46.4-83.2-22.618-96.8-26.618s-11.2,5.6-14.4,6.4-42.4-24-48.8-23.2z");
        getChildren().add(pg66);
        SVGPath pg70 = new SVGPath();
        pg70.setFill(Color.web("#ea8c4d"));
        pg70.setContent("m134.82,27.091c-6.4,0.8-31.14-23.229-16.8,8.8,16.2,36.201-58.401,39.201-75.201,28.001s7.2,18.4,7.2,18.4c18.4,19.998-16,3.2-16,3.2-34.4-12.8-58.401,12.8-61.601,13.6s-8,3.998-8.8-2.4c-0.8-6.4-8.048-22.767-40,3.2-22.473,18.088-35.658-2.818-35.658-2.818l-3.6,8.616c-23.8-38.998-9.25,20.02-9.25,20.02,8.8,34.4,139.71-5.82,139.71-5.82s168.8-30.398,180-34.398,104.65,2.254,104.65,2.254l-5.45-17.818c-64.8-46.4-82.4-22.037-96-26.037s-11.2,5.6-14.4,6.401c-3.2,0.8-42.4-24.001-48.8-23.201z");
        getChildren().add(pg70);
        SVGPath pg74 = new SVGPath();
        pg74.setFill(Color.web("#ec9961"));
        pg74.setContent("m136.13,28.837c-6.4,0.8-31.13-23.232-16.8,8.8,16.8,37.556-58.936,38.845-75.202,28-16.8-11.2,7.2,18.4,7.2,18.4,18.4,20.003-16,3.2-16,3.2-34.4-12.8-58.4,12.803-61.6,13.603s-8,4-8.8-2.403c-0.8-6.4-7.917-22.598-40.001,3.203-23.709,18.83-36.587-1.53-36.587-1.53l-4,7.13c-21.8-36.803-8.58,21.52-8.58,21.52,8.8,34.4,140.37-7.12,140.37-7.12s168.8-30.403,180-34.403,103.78,2.182,103.78,2.182l-5.38-18.327c-64.8-46.401-81.6-21.455-95.2-25.455s-11.2,5.6-14.4,6.4-42.4-24-48.8-23.2z");
        getChildren().add(pg74);
        SVGPath pg78 = new SVGPath();
        pg78.setFill(Color.web("#eea575"));
        pg78.setContent("m137.44,30.583c-6.4,0.8-30.63-23.454-16.8,8.8,16.8,39.2-58.403,39.2-75.203,28s7.2,18.4,7.2,18.4c18.4,19.997-16,3.2-16,3.2-34.4-12.8-58.4,12.797-61.6,13.597s-8,4-8.8-2.4c-0.8-6.397-7.785-22.428-40,3.2-24.946,19.58-37.507-0.23-37.507-0.23l-4.4,5.63c-19.8-34.798-7.91,23.04-7.91,23.04,8.8,34.4,141.02-8.44,141.02-8.44s168.8-30.397,180-34.397,102.91,2.109,102.91,2.109l-5.31-18.837c-64.8-46.4-80.8-20.872-94.4-24.872s-11.2,5.6-14.4,6.4-42.4-24-48.8-23.2z");
        getChildren().add(pg78);
        SVGPath pg82 = new SVGPath();
        pg82.setFill(Color.web("#f1b288"));
        pg82.setContent("m138.75,32.328c-6.4,0.8-32.37-22.651-16.8,8.8,19.2,38.8-58.404,39.2-75.204,28s7.2,18.4,7.2,18.4c18.4,20.002-16,3.2-16,3.2-34.4-12.8-58.4,12.802-61.6,13.602s-8,4-8.8-2.4c-0.8-6.402-7.654-22.265-40,3.2-26.182,20.33-38.436,1.05-38.436,1.05l-4.8,4.15c-18-33.202-7.24,24.54-7.24,24.54,8.8,34.4,141.68-9.74,141.68-9.74s168.8-30.402,180-34.402,102.03,2.036,102.03,2.036l-5.23-19.345c-64.8-46.4-80-20.291-93.6-24.291s-11.2,5.6-14.4,6.4-42.4-24-48.8-23.2z");
        getChildren().add(pg82);
        SVGPath pg86 = new SVGPath();
        pg86.setFill(Color.web("#f3bf9c"));
        pg86.setContent("m140.06,34.073c-6.4,0.8-32.75-22.46-16.8,8.8,20.4,40.001-58.405,39.201-75.205,28.001s7.2,18.4,7.2,18.4c18.4,19.996-16,3.2-16,3.2-34.4-12.8-58.4,12.796-61.6,13.596s-8,4-8.8-2.4c-0.8-6.396-7.523-22.092-40,3.2-27.419,21.08-39.365,2.35-39.365,2.35l-5.2,2.65c-16-30.196-6.56,26.06-6.56,26.06,8.8,34.4,142.32-11.06,142.32-11.06s168.8-30.396,180-34.396,101.16,1.963,101.16,1.963l-5.16-19.854c-64.8-46.4-79.2-19.709-92.8-23.709-13.6-4.001-11.2,5.6-14.4,6.4s-42.4-24.001-48.8-23.201z");
        getChildren().add(pg86);
        SVGPath pg90 = new SVGPath();
        pg90.setFill(Color.web("#f5ccb0"));
        pg90.setContent("m141.36,35.819c-6.4,0.8-33.84-21.875-16.8,8.8,22,39.6-58.396,39.2-75.196,28s7.2,18.4,7.2,18.4c18.4,20.001-16,3.2-16,3.2-34.4-12.8-58.4,12.801-61.6,13.601s-8,4-8.8-2.4c-0.8-6.401-7.391-21.928-40,3.2-28.655,21.82-40.294,3.64-40.294,3.64l-5.6,1.16c-14.4-28.401-5.89,27.56-5.89,27.56,8.8,34.4,142.98-12.36,142.98-12.36s168.8-30.401,180-34.401,100.3,1.891,100.3,1.891l-5.1-20.364c-64.8-46.4-78.4-19.127-92-23.127s-11.2,5.6-14.4,6.4-42.4-24-48.8-23.2z");
        getChildren().add(pg90);
        SVGPath pg94 = new SVGPath();
        pg94.setFill(Color.web("#f8d8c4"));
        pg94.setContent("m142.67,37.565c-6.4,0.8-33.84-21.876-16.8,8.8,22,39.6-58.396,39.2-75.196,28s7.2,18.4,7.2,18.4c18.4,19.995-16,3.2-16,3.2-34.401-12.8-58.401,12.795-61.601,13.595s-8,4-8.8-2.4-7.259-21.755-40,3.2c-29.891,22.57-41.213,4.93-41.213,4.93l-6-0.33c-13.61-26.396-5.22,29.08-5.22,29.08,8.8,34.4,143.63-13.68,143.63-13.68s168.8-30.395,180-34.395,99.42,1.818,99.42,1.818l-5.01-20.873c-64.81-46.4-77.61-18.545-91.21-22.545s-11.2,5.6-14.4,6.4-42.4-24-48.8-23.2z");
        getChildren().add(pg94);
        SVGPath pg98 = new SVGPath();
        pg98.setFill(Color.web("#fae5d7"));
        pg98.setContent("m143.98,39.31c-6.4,0.8-33.45-22.087-16.8,8.8,22,40.8-58.397,39.2-75.197,28s7.2,18.4,7.2,18.4c18.4,20-16,3.2-16,3.2-34.4-12.8-58.4,12.8-61.6,13.6-3.201,0.8-8.001,4-8.801-2.4s-7.128-21.592-40,3.2c-31.127,23.31-42.142,6.22-42.142,6.22l-6.4-1.82c-13-24-4.55,30.58-4.55,30.58,8.8,34.4,144.29-14.98,144.29-14.98s168.8-30.4,180-34.4,98.55,1.746,98.55,1.746l-4.95-21.382c-64.8-46.401-76.8-17.964-90.4-21.964s-11.2,5.6-14.4,6.4-42.4-24-48.8-23.2z");
        getChildren().add(pg98);
        SVGPath pg102 = new SVGPath();
        pg102.setFill(Color.web("#fcf2eb"));
        pg102.setContent("m145.29,41.055c-6.4,0.8-32.37-22.644-16.8,8.8,21.2,42.801-58.398,39.201-75.198,28.001s7.2,18.4,7.2,18.4c18.4,20.004-16,3.2-16,3.2-34.4-12.8-58.4,12.804-61.6,13.604s-8,4-8.8-2.4-6.997-21.428-40,3.2c-32.365,24.05-43.072,7.5-43.072,7.5l-6.8-3.3c-12.8-23.204-3.87,32.09-3.87,32.09,8.8,34.4,144.94-16.29,144.94-16.29s168.8-30.4,180-34.404c11.2-4,97.67,1.674,97.67,1.674l-4.87-21.893c-64.8-46.4-76-17.381-89.6-21.381-13.6-4.001-11.2,5.6-14.4,6.4s-42.4-24.001-48.8-23.201z");
        getChildren().add(pg102);
        SVGPath pg106 = new SVGPath();
        pg106.setFill(Color.WHITE);
        pg106.setContent("m-115.8,119.6c-12.8-22-3.2,33.6-3.2,33.6,8.8,34.4,145.6-17.6,145.6-17.6s168.8-30.4,180-34.4,96.8,1.6,96.8,1.6l-4.8-22.4c-64.8-46.4-75.2-16.8-88.8-20.8s-11.2,5.6-14.4,6.4-42.4-24-48.8-23.2-31.62-23.007-16.8,8.8c22.23,47.707-60.759,37.627-75.2,28-16.8-11.2,7.2,18.4,7.2,18.4,18.4,20-16,3.2-16,3.2-34.4-12.8-58.4,12.8-61.6,13.6s-8,4-8.8-2.4-6.865-21.256-40,3.2c-33.6,24.8-44,8.8-44,8.8l-7.2-4.8z");
        getChildren().add(pg106);
        SVGPath pg110 = new SVGPath();
        pg110.setFill(Color.BLACK);
        pg110.setContent("m-74.2,149.6s-7.2,11.6,13.6,24.8c0,0,1.4,1.4-16.6-2.8,0,0-6.2-2-7.8-12.4,0,0-4.8-4.4-9.6-10s20.4,0.4,20.4,0.4z");
        getChildren().add(pg110);
        SVGPath pg114 = new SVGPath();
        pg114.setFill(Color.web("#CCC"));
        pg114.setContent("m65.8,102s17.698,26.82,17.1,31.6c-1.3,10.4-1.5,20,1.7,24,3.201,4,12.001,37.2,12.001,37.2s-0.4,1.2,11.999-36.8c0,0,11.6-16-8.4-34.4,0,0-35.2-28.8-34.4-21.6z");
        getChildren().add(pg114);
        SVGPath pg118 = new SVGPath();
        pg118.setFill(Color.BLACK);
        pg118.setContent("m-54.2,176.4s11.2,7.2-3.2,38.4l6.4-2.4s-0.8,11.2-4,13.6l7.2-3.2s4.8,8,0.8,12.8c0,0,16.8,8,16,14.4,0,0,6.4-8,2.4-14.4s-11.2-2.4-10.4-20.8l-8.8,3.2s5.6-8.8,5.6-15.2l-8,2.4s15.469-26.58,4.8-28c-6-0.8-8.8-0.8-8.8-0.8z");
        getChildren().add(pg118);
        SVGPath pg122 = new SVGPath();
        pg122.setFill(Color.web("#CCC"));
        pg122.setContent("m-21.8,193.2s2.8-4.4,0-3.6-34,15.6-40,25.2c0,0,34.4-24.4,40-21.6z");
        getChildren().add(pg122);
        SVGPath pg126 = new SVGPath();
        pg126.setFill(Color.web("#CCC"));
        pg126.setContent("m-11.4,201.2s2.8-4.4,0-3.6-34,15.6-40,25.2c0,0,34.4-24.4,40-21.6z");
        getChildren().add(pg126);
        SVGPath pg130 = new SVGPath();
        pg130.setFill(Color.web("#CCC"));
        pg130.setContent("m1.8,186s2.8-4.4,0-3.6-34,15.6-40,25.2c0,0,34.4-24.4,40-21.6z");
        getChildren().add(pg130);
        SVGPath pg134 = new SVGPath();
        pg134.setFill(Color.web("#CCC"));
        pg134.setContent("m-21.4,229.6s0-6-2.8-5.2-38.8,18.4-44.8,28c0,0,42-25.6,47.6-22.8z");
        getChildren().add(pg134);
        SVGPath pg138 = new SVGPath();
        pg138.setFill(Color.web("#CCC"));
        pg138.setContent("m-20.2,218.8s1.2-4.8-1.6-4c-2,0-28.4,11.6-34.4,21.2,0,0,29.6-21.6,36-17.2z");
        getChildren().add(pg138);
        SVGPath pg142 = new SVGPath();
        pg142.setFill(Color.web("#CCC"));
        pg142.setContent("m-34.6,266.4-10,7.6s10.4-7.6,14-6.4c0,0-6.8,11.2-7.6,16.4,0,0,10.4-12.8,16-12.4,0,0,7.6,0.4,7.6,11.2,0,0,5.6-10.4,8.8-10,0,0,1.2,6.4,0,13.2,0,0,4-7.6,8-6,0,0,6.4-2,5.6,9.6,0,0,0,10.4-0.8,13.2,0,0,5.6-26.4,8-26.8,0,0,8-1.2,12.8,7.6,0,0-4-7.6,0.8-5.6,0,0,10.8,1.6,14,8.4,0,0-6.8-12-1.2-8.8l8,6.4s8.4,21.2,10.4,22.8c0,0-7.6-21.6-6-21.6,0,0-2-12,3.2,2.8,0,0-3.2-14,2.4-13.2s10,10.8,18.4,8.4c0,0,9.601,5.6,11.601-63.6l-124,46.8z");
        getChildren().add(pg142);
        SVGPath pg146 = new SVGPath();
        pg146.setFill(Color.BLACK);
        pg146.setContent("m-29.8,173.6s14.8-6,54.8,0c0,0,7.2,0.4,14-8.4s33.6-16,40-14l9.601,6.4,0.8,1.2s12.399,10.4,12.799,18-14.399,55.6-24,71.6c-9.6,16-19.2,28.4-38.4,26,0,0-20.8-4-46.4,0,0,0-29.2-1.6-32-9.6s11.2-23.2,11.2-23.2,4.4-8.4,3.2-22.8-0.8-42.4-5.6-45.2z");
        getChildren().add(pg146);
        SVGPath pg150 = new SVGPath();
        pg150.setFill(Color.web("#e5668c"));
        pg150.setContent("M-7.8,175.6c8.4,18.4-21.2,83.6-21.2,83.6-2,1.6,12.66,7.65,22.8,5.2,10.946-2.64,51.2,1.6,51.2,1.6,23.6-15.6,36.4-60,36.4-60s10.401-24-7.2-27.2c-17.6-3.2-82-3.2-82-3.2z");
        getChildren().add(pg150);
        SVGPath pg154 = new SVGPath();
        pg154.setFill(Color.web("#b23259"));
        pg154.setContent("m-9.831,206.5c3.326-12.79,4.91-24.59,2.031-30.9,0,0,62.4,6.4,73.6-14.4,4.241-7.87,19.001,22.8,18.6,32.4,0,0-63,14.4-77.8,3.2l-16.431,9.7z");
        getChildren().add(pg154);
        SVGPath pg158 = new SVGPath();
        pg158.setFill(Color.web("#a5264c"));
        pg158.setContent("m-5.4,222.8s2,7.2-0.4,11.2c0,0-1.6,0.8-2.8,1.2,0,0,1.2,3.6,7.2,5.2,0,0,2,4.4,4.4,4.8s7.2,6,11.2,4.8,15.2-5.2,15.2-5.2,5.6-3.2,14.4,0.4c0,0,2.375-0.8,2.8-4.8,0.5-4.7,3.6-8.4,5.6-10.4s11.6-14.8,10.4-15.2-68,8-68,8z");
        getChildren().add(pg158);
        SVGPath pg162 = new SVGPath();
        pg162.setStroke(Color.web("#000"));
        pg162.setFill(Color.web("#ff727f"));
        pg162.setContent("m-9.8,174.4s-2.8,22.4,0.4,30.8,2.4,10.4,1.6,14.4,3.6,14,9.2,20l12,1.6s15.2-3.6,24.4-0.8c0,0,8.994,1.34,12.4-13.6,0,0,4.8-6.4,12-9.2s14.4-44.4,10.4-52.4-18.4-12.4-34.4,3.2-18-1.2-48,6z");
        getChildren().add(pg162);
        SVGPath pg166 = new SVGPath();
        pg166.setStrokeWidth(0.5);
        pg166.setStroke(Color.web("#000"));
        pg166.setFill(Color.web("#FFC"));
        pg166.setContent("m-8.2,249.2s-0.8-2-5.2-2.4c0,0-22.4-3.6-30.8-16,0,0-6.8-5.6-2.4,6,0,0,10.4,20.4,17.2,23.2,0,0,16.4,4,21.2-10.8z");
        getChildren().add(pg166);
        SVGPath pg170 = new SVGPath();
        pg170.setFill(Color.web("#cc3f4c"));
        pg170.setContent("m71.742,185.23c0.659-7.91,2.612-16.52,0.858-20.03-6.446-12.89-23.419-7.5-34.4,3.2-16,15.6-18-1.2-48,6,0,0-1.745,13.96-0.905,23.98,0,0,37.305-11.58,38.105-5.98,0,0,1.6-3.2,10.8-3.2s31.942-1.17,33.542-3.97z");
        getChildren().add(pg170);
        SVGPath pg174 = new SVGPath();
        pg174.setStrokeWidth(2.0);
        pg174.setStroke(Color.web("#a51926"));
        pg174.setContent("m28.6,175.2s4.8,4.8,1.2,14.4c0,0-14.4,16-12.4,30");
        getChildren().add(pg174);
        SVGPath pg178 = new SVGPath();
        pg178.setStrokeWidth(0.5);
        pg178.setStroke(Color.web("#000"));
        pg178.setFill(Color.web("#FFC"));
        pg178.setContent("m-19.4,260s-4.4-12.8,4.4-6l3.6,3.6c-1.2,1.6-6.8,5.6-8,2.4z");
        getChildren().add(pg178);
        SVGPath pg182 = new SVGPath();
        pg182.setStrokeWidth(0.5);
        pg182.setStroke(Color.web("#000"));
        pg182.setFill(Color.web("#FFC"));
        pg182.setContent("m-14.36,261.2s-3.52-10.24,3.52-4.8l2.88,2.88c-4.56,1.28,0,3.84-6.4,1.92z");
        getChildren().add(pg182);
        SVGPath pg186 = new SVGPath();
        pg186.setStrokeWidth(0.5);
        pg186.setStroke(Color.web("#000"));
        pg186.setFill(Color.web("#FFC"));
        pg186.setContent("m-9.56,261.2s-3.52-10.24,3.52-4.8l2.88,2.88c-3.36,1.28,0,3.84-6.4,1.92z");
        getChildren().add(pg186);
        SVGPath pg190 = new SVGPath();
        pg190.setStrokeWidth(0.5);
        pg190.setStroke(Color.web("#000"));
        pg190.setFill(Color.web("#FFC"));
        pg190.setContent("m-2.96,261.4s-3.52-10.24,3.52-4.8c0,0,4.383,2.33,2.881,2.88-2.961,1.08,0,3.84-6.401,1.92z");
        getChildren().add(pg190);
        SVGPath pg194 = new SVGPath();
        pg194.setStrokeWidth(0.5);
        pg194.setStroke(Color.web("#000"));
        pg194.setFill(Color.web("#FFC"));
        pg194.setContent("m3.52,261.32s-3.52-10.24,3.521-4.8l2.88,2.88c-0.96,1.28,0,3.84-6.401,1.92z");
        getChildren().add(pg194);
        SVGPath pg198 = new SVGPath();
        pg198.setStrokeWidth(0.5);
        pg198.setStroke(Color.web("#000"));
        pg198.setFill(Color.web("#FFC"));
        pg198.setContent("m10.2,262s-4.8-12.4,4.4-6l3.6,3.6c-1.2,1.6,0,4.8-8,2.4z");
        getChildren().add(pg198);
        SVGPath pg202 = new SVGPath();
        pg202.setStrokeWidth(2.0);
        pg202.setStroke(Color.web("#a5264c"));
        pg202.setContent("m-18.2,244.8s13.2-2.8,19.2,0.4c0,0,6,1.2,7.2,0.8s4.4-0.8,4.4-0.8");
        getChildren().add(pg202);
        SVGPath pg206 = new SVGPath();
        pg206.setStrokeWidth(2.0);
        pg206.setStroke(Color.web("#a5264c"));
        pg206.setContent("m15.8,253.6s12-13.6,24-9.2c7.016,2.57,6-0.8,6.8-3.6s1-7,6-10");
        getChildren().add(pg206);
        SVGPath pg210 = new SVGPath();
        pg210.setStrokeWidth(0.5);
        pg210.setStroke(Color.web("#000"));
        pg210.setFill(Color.web("#FFC"));
        pg210.setContent("m33,237.6s-4-10.8-6.8,2-6,16.4-7.6,19.2c0,0,0,5.2,8.4,4.8,0,0,10.8-0.4,11.2-3.2s-1.2-14.4-5.2-22.8z");
        getChildren().add(pg210);
        SVGPath pg214 = new SVGPath();
        pg214.setStrokeWidth(2.0);
        pg214.setStroke(Color.web("#a5264c"));
        pg214.setContent("m47,244.8s3.6-2.4,6-1.2");
        getChildren().add(pg214);
        SVGPath pg218 = new SVGPath();
        pg218.setStrokeWidth(2.0);
        pg218.setStroke(Color.web("#a5264c"));
        pg218.setContent("m53.5,228.4s2.9-4.9,7.7-5.7");
        getChildren().add(pg218);
        SVGPath pg222 = new SVGPath();
        pg222.setFill(Color.web("#b2b2b2"));
        pg222.setContent("m-25.8,265.2s18,3.2,22.4,1.6l0.4,2-20.8-1.2s-11.6-5.6-2-2.4z");
        getChildren().add(pg222);
        SVGPath pg226 = new SVGPath();
        pg226.setStrokeWidth(0.5);
        pg226.setStroke(Color.web("#000"));
        pg226.setFill(Color.web("#FFC"));
        pg226.setContent("m-11.8,172,19.6,0.8s7.2,30.8,3.6,38.4c0,0-1.2,2.8-4-2.8,0,0-18.4-32.8-21.6-34.8s1.2-1.6,2.4-1.6z");
        getChildren().add(pg226);
        SVGPath pg230 = new SVGPath();
        pg230.setStrokeWidth(0.5);
        pg230.setStroke(Color.web("#000"));
        pg230.setFill(Color.web("#FFC"));
        pg230.setContent("m-88.9,169.3s8.9,1.7,21.5,4.3c0,0,4.8,22.4,8,27.2s-0.4,4.8-4,2-18.4-16.8-20.4-21.2-5.1-12.3-5.1-12.3z");
        getChildren().add(pg230);
        SVGPath pg234 = new SVGPath();
        pg234.setStrokeWidth(0.5);
        pg234.setStroke(Color.web("#000"));
        pg234.setFill(Color.web("#FFC"));
        pg234.setContent("m-67.039,173.82s5.8,1.55,6.809,3.76c1.008,2.22-1.202,5.51-1.202,5.51s-1,3.31-2.202,1.15c-1.202-2.17-4.074-9.83-3.405-10.42z");
        getChildren().add(pg234);
        SVGPath pg238 = new SVGPath();
        pg238.setFill(Color.BLACK);
        pg238.setContent("m-67,173.6s3.6,5.2,7.2,5.2,3.982-0.41,6.8,0.2c4.6,1,4.2-1,10.8,0.2,2.64,0.48,5.2-0.4,8,0.8s6,0.4,7.2-1.6,6-6.2,6-6.2-12.8,1.8-15.6,2.6c0,0-22.4,1.2-30.4-1.2z");
        getChildren().add(pg238);
        SVGPath pg242 = new SVGPath();
        pg242.setStrokeWidth(0.5);
        pg242.setStroke(Color.web("#000"));
        pg242.setFill(Color.web("#FFC"));
        pg242.setContent("m-22.4,173.8s-6.45,3.5-6.85,5.9,5.25,6.1,5.25,6.1,2.75,4.6,3.35,2.2-0.95-13.8-1.75-14.2z");
        getChildren().add(pg242);
        SVGPath pg246 = new SVGPath();
        pg246.setStrokeWidth(0.5);
        pg246.setStroke(Color.web("#000"));
        pg246.setFill(Color.web("#FFC"));
        pg246.setContent("m-59.885,179.26s7.007,11.19,7.224-0.02c0,0,0.557-1.26-1.203-1.28-6.075-0.07-4.554-4.18-6.021,1.3z");
        getChildren().add(pg246);
        SVGPath pg250 = new SVGPath();
        pg250.setStrokeWidth(0.5);
        pg250.setStroke(Color.web("#000"));
        pg250.setFill(Color.web("#FFC"));
        pg250.setContent("m-52.707,179.51s7.921,11.19,7.285-0.09c0,0,0.007-0.33-1.746-0.48-4.747-0.42-4.402-4.94-5.539,0.57z");
        getChildren().add(pg250);
        SVGPath pg254 = new SVGPath();
        pg254.setStrokeWidth(0.5);
        pg254.setStroke(Color.web("#000"));
        pg254.setFill(Color.web("#FFC"));
        pg254.setContent("m-45.494,179.52s7.96,10.63,7.291,0.96c0,0,0.119-1.23-1.535-1.53-3.892-0.71-4.103-3.95-5.756,0.57z");
        getChildren().add(pg254);
        SVGPath pg258 = new SVGPath();
        pg258.setStrokeWidth(0.5);
        pg258.setStroke(Color.web("#000"));
        pg258.setFill(Color.web("#FFC"));
        pg258.setContent("m-38.618,179.6s7.9,11.56,8.248,1.78c0,0,1.644-1.38-0.102-1.6-5.818-0.74-5.02-5.19-8.146-0.18z");
        getChildren().add(pg258);
        SVGPath pg262 = new SVGPath();
        pg262.setFill(Color.web("#e5e5b2"));
        pg262.setContent("m-74.792,183.13-7.658-1.53c-2.6-5-4.7-11.15-4.7-11.15s6.35,1,18.85,3.8c0,0,0.876,3.32,2.348,9.11l-8.84-0.23z");
        getChildren().add(pg262);
        SVGPath pg266 = new SVGPath();
        pg266.setFill(Color.web("#e5e5b2"));
        pg266.setContent("m-9.724,178.47c-1.666-2.51-2.983-4.26-3.633-4.67-3.013-1.88,1.13-1.51,2.259-1.51l18.454,0.76s0.524,2.24,1.208,5.63c0,0-10.088-2.01-18.288-0.21z");
        getChildren().add(pg266);
        SVGPath pg270 = new SVGPath();
        pg270.setFill(Color.web("#cc7226"));
        pg270.setContent("m43.88,40.321c27.721,3.96,53.241-31.68,55.001-41.361,1.759-9.68-8.36-21.56-8.36-21.56,1.32-3.08-3.52-17.16-8.8-26.4s-21.181-8.266-38.721-9.24c-15.84-0.88-34.32,22.44-35.64,24.2s4.84,40.041,6.16,45.761-1.32,32.12-1.32,32.12c34.24-9.1,3.96-7.48,31.68-3.52z");
        getChildren().add(pg270);
        SVGPath pg274 = new SVGPath();
        pg274.setFill(Color.web("#ea8e51"));
        pg274.setContent("m8.088-33.392c-1.296,1.728,4.752,39.313,6.048,44.929s-1.296,31.536-1.296,31.536c32.672-8.88,3.888-7.344,31.104-3.456,27.217,3.888,52.273-31.104,54.001-40.609,1.728-9.504-8.208-21.168-8.208-21.168,1.296-3.024-3.456-16.848-8.64-25.92s-20.795-8.115-38.017-9.072c-15.552-0.864-33.696,22.032-34.992,23.76z");
        getChildren().add(pg274);
        SVGPath pg278 = new SVGPath();
        pg278.setFill(Color.web("#efaa7c"));
        pg278.setContent("m8.816-32.744c-1.272,1.696,4.664,38.585,5.936,44.097s-1.272,30.952-1.272,30.952c31.404-9.16,3.816-7.208,30.528-3.392,26.713,3.816,51.305-30.528,53.001-39.857,1.696-9.328-8.056-20.776-8.056-20.776,1.272-2.968-3.392-16.536-8.48-25.44s-20.41-7.965-37.313-8.904c-15.264-0.848-33.072,21.624-34.344,23.32z");
        getChildren().add(pg278);
        SVGPath pg282 = new SVGPath();
        pg282.setFill(Color.web("#f4c6a8"));
        pg282.setContent("m9.544-32.096c-1.248,1.664,4.576,37.857,5.824,43.265s-1.248,30.368-1.248,30.368c29.436-9.04,3.744-7.072,29.952-3.328,26.209,3.744,50.337-29.952,52.001-39.104,1.664-9.153-7.904-20.385-7.904-20.385,1.248-2.912-3.328-16.224-8.32-24.96s-20.025-7.815-36.609-8.736c-14.976-0.832-32.448,21.216-33.696,22.88z");
        getChildren().add(pg282);
        SVGPath pg286 = new SVGPath();
        pg286.setFill(Color.web("#f9e2d3"));
        pg286.setContent("m10.272-31.448c-1.224,1.632,4.488,37.129,5.712,42.433s-1.224,29.784-1.224,29.784c27.868-8.92,3.672-6.936,29.376-3.264,25.705,3.672,49.369-29.376,51.001-38.353,1.632-8.976-7.752-19.992-7.752-19.992,1.224-2.856-3.264-15.912-8.16-24.48s-19.64-7.665-35.905-8.568c-14.688-0.816-31.824,20.808-33.048,22.44z");
        getChildren().add(pg286);
        SVGPath pg290 = new SVGPath();
        pg290.setFill(Color.WHITE);
        pg290.setContent("M44.2,36.8c25.2,3.6,48.401-28.8,50.001-37.6s-7.6-19.6-7.6-19.6c1.2-2.8-3.201-15.6-8.001-24s-19.254-7.514-35.2-8.4c-14.4-0.8-31.2,20.4-32.4,22s4.4,36.4,5.6,41.6-1.2,29.2-1.2,29.2c25.5-8.6,3.6-6.8,28.8-3.2z");
        getChildren().add(pg290);
        SVGPath pg294 = new SVGPath();
        pg294.setFill(Color.web("#CCC"));
        pg294.setContent("m90.601,2.8s-27.801,7.6-39.401,6c0,0-15.8-6.6-24.6,15.2,0,0-3.6,7.2-5.6,9.2s69.601-30.4,69.601-30.4z");
        getChildren().add(pg294);
        SVGPath pg298 = new SVGPath();
        pg298.setFill(Color.BLACK);
        pg298.setContent("m94.401,0.6s-29.001,12.2-39.001,11.8c0,0-16.4-4.6-24.8,10,0,0-8.4,9.2-11.6,10.8,0,0-0.4,1.6,6-2.4l10.4,5.2s14.8,9.6,24.4-6.4c0,0,4-11.2,4-13.2s21.2-7.6,22.801-8c1.6-0.4,8.2-4.6,7.8-7.8z");
        getChildren().add(pg298);
        SVGPath pg302 = new SVGPath();
        pg302.setFill(Color.web("#99cc32"));
        pg302.setContent("m47,36.514c-6.872,0-15.245-3.865-15.245-10.114,0-6.248,8.373-12.513,15.245-12.513,6.874,0,12.446,5.065,12.446,11.313,0,6.249-5.572,11.314-12.446,11.314z");
        getChildren().add(pg302);
        SVGPath pg306 = new SVGPath();
        pg306.setFill(Color.web("#659900"));
        pg306.setContent("m43.377,19.83c-4.846,0.722-9.935,2.225-9.863,2.009,1.54-4.619,7.901-7.952,13.486-7.952,4.296,0,8.084,1.978,10.32,4.988,0,0-5.316-0.33-13.943,0.955z");
        getChildren().add(pg306);
        SVGPath pg310 = new SVGPath();
        pg310.setFill(Color.WHITE);
        pg310.setContent("m55.4,19.6s-4.4-3.2-4.4-1c0,0,3.6,4.4,4.4,1z");
        getChildren().add(pg310);
        SVGPath pg314 = new SVGPath();
        pg314.setFill(Color.BLACK);
        pg314.setContent("m45.4,27.726c-2.499,0-4.525-2.026-4.525-4.526,0-2.499,2.026-4.525,4.525-4.525,2.5,0,4.526,2.026,4.526,4.525,0,2.5-2.026,4.526-4.526,4.526z");
        getChildren().add(pg314);
        SVGPath pg318 = new SVGPath();
        pg318.setFill(Color.web("#cc7226"));
        pg318.setContent("m-58.6,14.4s-3.2-21.2-0.8-25.6c0,0,10.8-10,10.4-13.6,0,0-0.4-18-1.6-18.8s-8.8-6.8-14.8-0.4c0,0-10.4,18-9.6,24.4v2s-7.6-0.4-9.2,1.6c0,0-1.2,5.2-2.4,5.6,0,0-2.8,2.4-0.8,5.2,0,0-2,2.4-1.6,6.4l7.6,4s2,14.4,12.8,19.6c4.836,2.329,8-4.4,10-10.4z");
        getChildren().add(pg318);
        SVGPath pg322 = new SVGPath();
        pg322.setFill(Color.WHITE);
        pg322.setContent("m-59.6,12.56s-2.88-19.08-0.72-23.04c0,0,9.72-9,9.36-12.24,0,0-0.36-16.2-1.44-16.92s-7.92-6.12-13.32-0.36c0,0-9.36,16.2-8.64,21.96v1.8s-6.84-0.36-8.28,1.44c0,0-1.08,4.68-2.16,5.04,0,0-2.52,2.16-0.72,4.68,0,0-1.8,2.16-1.44,5.76l6.84,3.6s1.8,12.96,11.52,17.64c4.352,2.095,7.2-3.96,9-9.36z");
        getChildren().add(pg322);
        SVGPath pg326 = new SVGPath();
        pg326.setFill(Color.web("#eb955c"));
        pg326.setContent("m-51.05-42.61c-1.09-0.86-8.58-6.63-14.43-0.39,0,0-10.14,17.55-9.36,23.79v1.95s-7.41-0.39-8.97,1.56c0,0-1.17,5.07-2.34,5.46,0,0-2.73,2.34-0.78,5.07,0,0-1.95,2.34-1.56,6.24l7.41,3.9s1.95,14.04,12.48,19.11c4.714,2.27,7.8-4.29,9.75-10.14,0,0-3.12-20.67-0.78-24.96,0,0,10.53-9.75,10.14-13.26,0,0-0.39-17.55-1.56-18.33z");
        getChildren().add(pg326);
        SVGPath pg330 = new SVGPath();
        pg330.setFill(Color.web("#f2b892"));
        pg330.setContent("m-51.5-41.62c-0.98-0.92-8.36-6.46-14.06-0.38,0,0-9.88,17.1-9.12,23.18v1.9s-7.22-0.38-8.74,1.52c0,0-1.14,4.94-2.28,5.32,0,0-2.66,2.28-0.76,4.94,0,0-1.9,2.28-1.52,6.08l7.22,3.8s1.9,13.68,12.16,18.62c4.594,2.212,7.6-4.18,9.5-9.88,0,0-3.04-20.14-0.76-24.32,0,0,10.26-9.5,9.88-12.92,0,0-0.38-17.1-1.52-17.86z");
        getChildren().add(pg330);
        SVGPath pg334 = new SVGPath();
        pg334.setFill(Color.web("#f8dcc8"));
        pg334.setContent("m-51.95-40.63c-0.87-0.98-8.14-6.29-13.69-0.37,0,0-9.62,16.65-8.88,22.57v1.85s-7.03-0.37-8.51,1.48c0,0-1.11,4.81-2.22,5.18,0,0-2.59,2.22-0.74,4.81,0,0-1.85,2.22-1.48,5.92l7.03,3.7s1.85,13.32,11.84,18.13c4.473,2.154,7.4-4.07,9.25-9.62,0,0-2.96-19.61-0.74-23.68,0,0,9.99-9.25,9.62-12.58,0,0-0.37-16.65-1.48-17.39z");
        getChildren().add(pg334);
        SVGPath pg338 = new SVGPath();
        pg338.setFill(Color.WHITE);
        pg338.setContent("m-59.6,12.46s-2.88-18.98-0.72-22.94c0,0,9.72-9,9.36-12.24,0,0-0.36-16.2-1.44-16.92-0.76-1.04-7.92-6.12-13.32-0.36,0,0-9.36,16.2-8.64,21.96v1.8s-6.84-0.36-8.28,1.44c0,0-1.08,4.68-2.16,5.04,0,0-2.52,2.16-0.72,4.68,0,0-1.8,2.16-1.44,5.76l6.84,3.6s1.8,12.96,11.52,17.64c4.352,2.095,7.2-4.06,9-9.46z");
        getChildren().add(pg338);
        SVGPath pg342 = new SVGPath();
        pg342.setFill(Color.web("#CCC"));
        pg342.setContent("m-62.7,6.2s-21.6-10.2-22.5-11c0,0,9.1,8.2,9.9,8.2s12.6,2.8,12.6,2.8z");
        getChildren().add(pg342);
        SVGPath pg346 = new SVGPath();
        pg346.setFill(Color.BLACK);
        pg346.setContent("m-79.8,0s18.4,3.6,18.4,8c0,2.912-0.243,16.331-5.6,14.8-8.4-2.4-4.8-16.8-12.8-22.8z");
        getChildren().add(pg346);
        SVGPath pg350 = new SVGPath();
        pg350.setFill(Color.web("#99cc32"));
        pg350.setContent("m-71.4,3.8s8.978,1.474,10,4.2c0.6,1.6,1.263,9.908-4.2,11-4.552,0.911-6.782-9.31-5.8-15.2z");
        getChildren().add(pg350);
        SVGPath pg354 = new SVGPath();
        pg354.setFill(Color.BLACK);
        pg354.setContent("m14.595,46.349c-0.497-1.742,0.814-1.611,2.605-2.149,2-0.6,14.2-4.4,15-7s14,1.8,14,1.8c1.8,0.8,6.2,3.4,6.2,3.4,4.8,1.2,11.4,1.6,11.4,1.6,2.4,1,5.8,3.8,5.8,3.8,14.6,10.2,27.001,3,27.001,3,19.999-6.6,13.999-23.8,13.999-23.8-3-9,0.2-12.4,0.2-12.4,0.2-3.8,7.4,2.6,7.4,2.6,2.6,4.2,3.4,9.2,3.4,9.2,8,11.2,4.6-6.6,4.6-6.6,0.2-1-2.6-4.6-2.6-5.8s-1.8-4.6-1.8-4.6c-3-3.4-0.6-10.4-0.6-10.4,1.8-13.8-0.4-12-0.4-12-1.2-1.8-10.4,8.2-10.4,8.2-2.2,3.4-8.2,5-8.2,5-2.799,1.8-6.199,0.4-6.199,0.4-2.6-0.4-8.2,6.6-8.2,6.6,2.8-0.2,5.2,4.2,7.6,4.4s4.2-2.4,5.799-3c1.6-0.6,4.4,5.2,4.4,5.2,0.4,2.6-5.2,7.4-5.2,7.4-0.4,4.6-1.999,3-1.999,3-3-0.6-4.2,3.2-5.2,7.8s-5.2,5-5.2,5c-1.6,7.4-2.801,4.4-2.801,4.4-0.2-5.6-6.2,0.2-6.2,0.2-1.2,2-5.8-0.2-5.8-0.2-6.8-2-4.4-4-4.4-4,1.8-2.2,13,0,13,0,2.2-1.6-5.8-5.6-5.8-5.6-0.6-1.8,0.4-6.2,0.4-6.2,1.2-3.2,8-8.8,8-8.8,9.401-1.2,6.601-2.8,6.601-2.8-6.2-5.2-12.001,2.4-12.001,2.4-2.2,6.2-19.6,21.2-19.6,21.2-4.8,3.4-2.2-3.4-6.2,0s-24.6-5.6-24.6-5.6c-11.562-1.193-14.294,14.549-17.823,11.429,0,0,5.418,8.52,3.818,2.92z");
        getChildren().add(pg354);
        SVGPath pg358 = new SVGPath();
        pg358.setFill(Color.BLACK);
        pg358.setContent("m209.4-120s-25.6,8-28.4,26.8c0,0-2.4,22.8,18,40.4,0,0,0.4,6.4,2.4,9.6,0,0-1.6,4.8,17.2-2.8l27.2-8.4s6.4-2.4,11.6-11.2,20.4-27.6,16.8-52.8c0,0,1.2-11.2-4.8-11.6,0,0-8.4-1.6-15.6,6,0,0-6.8,3.2-9.2,2.8l-35.2,1.2z");
        getChildren().add(pg358);
        SVGPath pg362 = new SVGPath();
        pg362.setFill(Color.BLACK);
        pg362.setContent("m264.02-120.99s2.1-8.93-2.74-4.09c0,0-7.04,5.72-14.52,5.72,0,0-14.52,2.2-18.92,15.4,0,0-3.96,26.84,3.96,32.56,0,0,4.84,7.48,11.88,0.88s22.54-36.83,20.34-50.47z");
        getChildren().add(pg362);
        SVGPath pg366 = new SVGPath();
        pg366.setFill(Color.web("#323232"));
        pg366.setContent("m263.65-120.63s2.09-8.75-2.66-3.99c0,0-6.92,5.61-14.26,5.61,0,0-14.26,2.16-18.58,15.12,0,0-3.89,26.354,3.89,31.97,0,0,4.75,7.344,11.66,0.864,6.92-6.48,22.11-36.184,19.95-49.574z");
        getChildren().add(pg366);
        SVGPath pg370 = new SVGPath();
        pg370.setFill(Color.web("#666"));
        pg370.setContent("m263.27-120.27s2.08-8.56-2.58-3.9c0,0-6.78,5.51-13.99,5.51,0,0-14,2.12-18.24,14.84,0,0-3.81,25.868,3.82,31.38,0,0,4.66,7.208,11.45,0.848,6.78-6.36,21.66-35.538,19.54-48.678z");
        getChildren().add(pg370);
        SVGPath pg374 = new SVGPath();
        pg374.setFill(Color.web("#999"));
        pg374.setContent("m262.9-119.92s2.07-8.37-2.51-3.79c0,0-6.65,5.41-13.73,5.41,0,0-13.72,2.08-17.88,14.56,0,0-3.75,25.372,3.74,30.78,0,0,4.58,7.072,11.23,0.832,6.66-6.24,21.23-34.892,19.15-47.792z");
        getChildren().add(pg374);
        SVGPath pg378 = new SVGPath();
        pg378.setFill(Color.web("#CCC"));
        pg378.setContent("m262.53-119.56s2.06-8.18-2.43-3.7c0,0-6.53,5.31-13.47,5.31,0,0-13.46,2.04-17.54,14.28,0,0-3.67,24.886,3.67,30.19,0,0,4.49,6.936,11.02,0.816,6.52-6.12,20.79-34.246,18.75-46.896z");
        getChildren().add(pg378);
        SVGPath pg382 = new SVGPath();
        pg382.setFill(Color.WHITE);
        pg382.setContent("m262.15-119.2s2.05-8-2.35-3.6c0,0-6.4,5.2-13.2,5.2,0,0-13.2,2-17.2,14,0,0-3.6,24.4,3.6,29.6,0,0,4.4,6.8,10.8,0.8s20.35-33.6,18.35-46z");
        getChildren().add(pg382);
        SVGPath pg386 = new SVGPath();
        pg386.setFill(Color.web("#992600"));
        pg386.setContent("m50.6,84s-20.4-19.2-28.4-20c0,0-34.4-4-49.2,14,0,0,17.6-20.4,45.2-14.8,0,0-21.6-4.4-34-1.2l-26.4,14-2.8,4.8s4-14.8,22.4-20.8c0,0,22.8-4.8,33.6,0,0,0-21.6-6.8-31.6-4.8,0,0-30.4-2.4-43.2,24,0,0,4-14.4,18.8-21.6,0,0,13.6-8.8,34-6,0,0,14.4,3.2,19.6,5.6s4-0.4-4.4-5.2c0,0-5.6-10-19.6-9.6,0,0-42.8,3.6-53.2,15.6,0,0,13.6-11.2,24-14,0,0,22.4-8,30.8-7.2,0,0,24.8,1,32.4-3,0,0-11.2,5-8,8.2s10,10.8,10,12,24.2,23.3,27.8,27.7l2.2,2.3z");
        getChildren().add(pg386);
        SVGPath pg390 = new SVGPath();
        pg390.setFill(Color.web("#CCC"));
        pg390.setContent("m189,278s-15.5-36.5-28-46c0,0,26,16,29.5,34,0,0,0,10-1.5,12z");
        getChildren().add(pg390);
        SVGPath pg394 = new SVGPath();
        pg394.setFill(Color.web("#CCC"));
        pg394.setContent("m236,285.5s-26.5-55-45-79c0,0,43.5,37.5,48.5,64l0.5,5.5-3-2.5s-0.5,9-1,12z");
        getChildren().add(pg394);
        SVGPath pg398 = new SVGPath();
        pg398.setFill(Color.web("#CCC"));
        pg398.setContent("m292.5,237s-62.5-59.5-64-62c0,0,60.5,66,63.5,73.5,0,0-2-9,0.5-11.5z");
        getChildren().add(pg398);
        SVGPath pg402 = new SVGPath();
        pg402.setFill(Color.web("#CCC"));
        pg402.setContent("m104,280.5s19.5-52,38.5-29.5c0,0,15,10,14.5,13,0,0-4-6.5-22-6,0,0-19-3-31,22.5z");
        getChildren().add(pg402);
        SVGPath pg406 = new SVGPath();
        pg406.setFill(Color.web("#CCC"));
        pg406.setContent("m294.5,153s-45-28.5-52.5-30c-11.81-2.36,49.5,29,54.5,39.5,0,0,2-2.5-2-9.5z");
        getChildren().add(pg406);
        SVGPath pg410 = new SVGPath();
        pg410.setFill(Color.BLACK);
        pg410.setContent("m143.8,259.6s20.4-2,27.2-8.8l4.4,3.6,17.6-38.4,3.6,5.2s14.4-14.8,13.6-22.8,12.8,6,12.8,6-0.8-11.6,6.4-4.8c0,0-2.4-15.6,6-7.6,0,0-10.54-30.16,12-4.4,5.6,6.4,1.2-0.4,1.2-0.4s-26-48-4.4-33.6c0,0,2-22.8,0.8-27.2s-3.2-26.8-8-32,0.4-6.8,6-1.6c0,0-11.2-24,2-12,0,0-3.6-15.2-8-18,0,0-5.6-17.2,9.6-6.4,0,0-4.4-12.4-7.6-15.6,0,0-11.6-27.6-4.4-22.8l4.4,3.6s-6.8-14-0.4-9.6,6.4,4,6.4,4-21.2-33.2-0.8-15.6c0,0-8.16-13.918-11.6-20.8,0,0-18.8-20.4-4.4-14l4.8,1.6s-8.8-10-16.8-11.6,2.4-8,8.8-6,22,9.6,22,9.6,12.8,18.8,16.8,19.2c0,0-20-7.6-14,0.4,0,0,14.4,14,7.2,13.6,0,0-6,7.2-1.2,16,0,0-18.46-18.391-3.6,7.2l6.8,16.4s-24.4-24.8-13.2-2.8c0,0,17.2,23.6,19.2,24s6.4,9.2,6.4,9.2l-4.4-2,5.2,8.8s-11.2-12-5.2,1.2l5.6,14.4s-20.4-22-6.8,7.6c0,0-16.4-5.2-7.6,12,0,0-1.6,16-1.2,21.2s1.6,33.6-2.8,41.6,6,27.2,8,31.2,5.6,14.8-3.2,5.6-4.4-3.6-2.4,5.2,8,24.4,7.2,30c0,0-1.2,1.2-4.4-2.4,0,0-14.8-22.8-13.2-8.4,0,0-1.2,8-4.4,16.8,0,0-3.2,10.8-3.2,2,0,0-3.2-16.8-6-9.2s-6.4,13.6-9.2,16-8-20.4-9.2-10c0,0-12-12.4-16.8,4l-11.6,16.4s-0.4-12.4-1.6-6.4c0,0-30,6-40.4,1.6z");
        getChildren().add(pg410);
        SVGPath pg414 = new SVGPath();
        pg414.setFill(Color.BLACK);
        pg414.setContent("m109.4-97.2s-11.599-8-15.599-7.6,27.599-8.8,68.799,18.8c0,0,4.8,2.8,8.4,2.4,0,0,3.2,2.4,0.4,6,0,0-8.8,9.6,2.4,20.8,0,0,18.4,6.8,12.8-2,0,0,10.8,4,13.2,8s1.2,0,1.2,0l-12.4-12.4s-5.2-2-8-10.4-5.2-18.4-0.8-21.6c0,0-4,4.4-3.2,0.4s4.4-7.6,6-8,18-16.2,24.8-16.6c0,0-9.2,1.4-12.2,0.4s-29.6-12.4-35.6-13.6c0,0-16.8-6.6-4.8-4.6,0,0,35.8,3.8,54,17,0,0-7.2-8.4-25.6-15.4,0,0-22.2-12.6-57.4-7.6,0,0-17.8,3.2-25.6,5,0,0-2.599-0.6-3.199-1s-12.401-9.4-40.001-2.4c0,0-17,4.6-25.6,9.4,0,0-15.2,1.2-18.8,4.4,0,0-18.6,14.6-20.6,15.4s-13.4,8.4-14.2,8.8c0,0,24.6-6.6,27-9s19.8-5,22.2-3.6,10.8,0.8,1.2,1.4c0,0,75.6,14.8,76.4,16.8s4.8,0.8,4.8,0.8z");
        getChildren().add(pg414);
        SVGPath pg418 = new SVGPath();
        pg418.setFill(Color.web("#cc7226"));
        pg418.setContent("m180.8-106.4s-10.2-7.4-12.2-7.4-14.4-10.2-18.6-9.8-16.4-9.6-43.8-1.4c0,0-0.6-2,3-2.8,0,0,6.4-2.2,6.8-2.8,0,0,20.2-4.2,27.4-0.6,0,0,9.2,2.6,15.4,8.8,0,0,11.2,3.2,14.4,2.2,0,0,8.8,2.2,9.2,4,0,0,5.8,3,4,5.6,0,0,0.4,1.6-5.6,4.2z");
        getChildren().add(pg418);
        SVGPath pg422 = new SVGPath();
        pg422.setFill(Color.web("#cc7226"));
        pg422.setContent("m168.33-108.51c0.81,0.63,1.83,0.73,2.43,1.54,0.24,0.31-0.05,0.64-0.37,0.74-1.04,0.31-2.1-0.26-3.24,0.33-0.4,0.21-1.04,0.03-1.6-0.12-1.63-0.44-3.46-0.47-5.15,0.22-1.98-1.13-4.34-0.54-6.42-1.55-0.06-0.02-0.28,0.32-0.36,0.3-3.04-1.15-6.79-0.87-9.22-3.15-2.43-0.41-4.78-0.87-7.21-1.55-1.82-0.51-3.23-1.5-4.85-2.33-1.38-0.71-2.83-1.23-4.37-1.61-1.86-0.45-3.69-0.34-5.58-0.86-0.1-0.02-0.29,0.32-0.37,0.3-0.32-0.11-0.62-0.69-0.79-0.64-1.68,0.52-3.17-0.45-4.83-0.11-1.18-1.22-2.9-0.98-4.45-1.42-2.97-0.85-6.12,0.42-9.15-0.58,4.11-1.84,8.8-0.61,12.86-2.68,2.33-1.18,4.99-0.08,7.56-0.84,0.49-0.15,1.18-0.35,1.58,0.32,0.14-0.14,0.32-0.37,0.38-0.35,2.44,1.16,4.76,2.43,7.24,3.5,0.34,0.15,0.88-0.09,1.13,0.12,1.52,1.21,3.46,1.11,4.85,2.33,1.7-0.5,3.49-0.12,5.22-0.75,0.08-0.02,0.31,0.32,0.34,0.3,1.14-0.75,2.29-0.48,3.18-0.18,0.34,0.12,1,0.37,1.31,0.44,1.12,0.27,1.98,0.75,3.16,0.94,0.11,0.02,0.3-0.32,0.37-0.3,1.12,0.44,2.16,0.39,2.82,1.55,0.14-0.14,0.3-0.37,0.38-0.35,1.03,0.34,1.68,1.1,2.78,1.34,0.48,0.1,1.1,0.73,1.67,0.91,2.39,0.73,4.24,2.26,6.43,3.15,0.76,0.31,1.64,0.55,2.27,1.04z");
        getChildren().add(pg422);
        SVGPath pg426 = new SVGPath();
        pg426.setFill(Color.web("#cc7226"));
        pg426.setContent("m91.696-122.74c-2.518-1.72-4.886-2.83-7.328-4.62-0.181-0.13-0.541,0.04-0.743-0.08-1.007-0.61-1.895-1.19-2.877-1.89-0.539-0.38-1.36-0.37-1.868-0.63-2.544-1.29-5.173-1.85-7.68-3.04,0.682-0.64,1.804-0.39,2.4-1.2,0.195,0.28,0.433,0.56,0.786,0.37,1.678-0.9,3.528-1.05,5.204-0.96,1.704,0.09,3.424,0.39,5.199,0.67,0.307,0.04,0.506,0.56,0.829,0.66,2.228,0.66,4.617,0.14,6.736,0.98,1.591,0.63,3.161,1.45,4.4,2.72,0.252,0.26-0.073,0.57-0.353,0.76,0.388-0.11,0.661,0.1,0.772,0.41,0.084,0.24,0.084,0.54,0,0.78-0.112,0.31-0.391,0.41-0.765,0.46-1.407,0.19,0.365-1.19-0.335-0.74-1.273,0.82-0.527,2.22-1.272,3.49-0.28-0.19-0.51-0.41-0.4-0.8,0.234,0.52-0.368,0.81-0.536,1.13-0.385,0.72-1.284,2.14-2.169,1.53z");
        getChildren().add(pg426);
        SVGPath pg430 = new SVGPath();
        pg430.setFill(Color.web("#cc7226"));
        pg430.setContent("m59.198-115.39c-3.154-0.79-6.204-0.68-9.22-1.96-0.067-0.02-0.29,0.32-0.354,0.3-1.366-0.6-2.284-1.56-3.36-2.61-0.913-0.89-2.571-0.5-3.845-0.99-0.324-0.12-0.527-0.63-0.828-0.67-1.219-0.16-2.146-1.11-3.191-1.68,2.336-0.8,4.747-0.76,7.209-1.15,0.113-0.02,0.258,0.31,0.391,0.31,0.136,0,0.266-0.23,0.4-0.36,0.195,0.28,0.497,0.61,0.754,0.35,0.548-0.54,1.104-0.35,1.644-0.31,0.144,0.01,0.269,0.32,0.402,0.32,0.136,0,0.267-0.32,0.4-0.32,0.136,0,0.267,0.32,0.4,0.32,0.136,0,0.266-0.23,0.4-0.36,0.692,0.78,1.577,0.23,2.399,0.41,1.038,0.22,1.305,1.37,2.379,1.67,4.715,1.3,8.852,3.45,13.215,5.54,0.307,0.14,0.517,0.39,0.407,0.78,0.267,0,0.58-0.09,0.77,0.04,1.058,0.74,2.099,1.28,2.796,2.38,0.216,0.34-0.113,0.75-0.346,0.7-4.429-1-8.435-1.61-12.822-2.71z");
        getChildren().add(pg430);
        SVGPath pg434 = new SVGPath();
        pg434.setFill(Color.web("#cc7226"));
        pg434.setContent("m45.338-71.179c-1.592-1.219-2.176-3.25-3.304-5.042-0.214-0.34,0.06-0.654,0.377-0.743,0.56-0.159,1.103,0.319,1.512,0.521,1.745,0.862,3.28,2.104,5.277,2.243,1.99,2.234,6.25,2.619,6.257,6,0.001,0.859-1.427-0.059-1.857,0.8-2.451-1.003-4.84-0.9-7.22-2.367-0.617-0.381-0.287-0.834-1.042-1.412z");
        getChildren().add(pg434);
        SVGPath pg438 = new SVGPath();
        pg438.setFill(Color.web("#cc7226"));
        pg438.setContent("m17.8-123.76c0.135,0,7.166,0.24,7.149,0.35-0.045,0.31-7.775,1.36-8.139,1.19-0.164-0.08-7.676,2.35-7.81,2.22,0.268-0.14,8.534-3.76,8.8-3.76z");
        getChildren().add(pg438);
        SVGPath pg442 = new SVGPath();
        pg442.setFill(Color.BLACK);
        pg442.setContent("m33.2-114s-14.8,1.8-19.2,3-23,8.8-26,10.8c0,0-13.4,5.4-30.4,25.4,0,0,7.6-3.4,9.8-6.2,0,0,13.6-12.6,13.4-10,0,0,12.2-8.6,11.6-6.4,0,0,24.4-11.2,22.4-8,0,0,21.6-4.6,20.6-2.6,0,0,18.8,4.4,16,4.6,0,0-5.8,1.2,0.6,4.8,0,0-3.4,4.4-8.8,0.4s-2.4-1.8-7.4-0.8c0,0-2.6,0.8-7.2-3.2,0,0-5.6-4.6-14.4-1,0,0-30.6,12.6-32.6,13.2,0,0-3.6,2.8-6,6.4,0,0-5.8,4.4-8.8,5.8,0,0-12.8,11.6-14,13,0,0-3.4,5.2-4.2,5.6,0,0,6.4-3.8,8.4-5.8,0,0,14-10,19.4-10.8,0,0,4.4-3,5.2-4.4,0,0,14.4-9.2,18.6-9.2,0,0,9.2,5.2,11.6-1.8,0,0,5.8-1.8,11.4-0.6,0,0,3.2-2.6,2.4-4.8,0,0,1.6-1.8,2.6,2,0,0,3.4,3.6,8.2,1.6,0,0,4-0.2,2,2.2,0,0-4.4,3.8-16.2,4,0,0-12.4,0.6-28.8,8.2,0,0-29.8,10.4-39,20.8,0,0-6.4,8.8-11.8,10,0,0-5.8,0.8-11.8,8.2,0,0,9.8-5.8,18.8-5.8,0,0,4-2.4,0.2,1.2,0,0-3.6,7.6-2,13,0,0-0.6,5.2-1.4,6.8,0,0-7.8,12.8-7.8,15.2s1.2,12.2,1.6,12.8-1-1.6,2.8,0.8,6.6,4,7.4,6.8-2-5.4-2.2-7.2-4.4-9-3.6-11.4c0,0,1,1,1.8,2.4,0,0-0.6-0.6,0-4.2,0,0,0.8-5.2,2.2-8.4s3.4-7,3.8-7.8,0.4-6.6,1.8-4l3.4,2.6s-2.8-2.6-0.6-4.8c0,0-1-5.6,0.8-8.2,0,0,7-8.4,8.6-9.4s0.2-0.6,0.2-0.6,6-4.2,0.2-2.6c0,0-4,1.6-7,1.6,0,0-7.6,2-3.6-2.2s14-9.6,17.8-9.4l0.8,1.6,11.2-2.4-1.2,0.8s-0.2-0.2,4-0.6,10,1,11.4-0.8,4.8-2.8,4.4-1.4-0.6,3.4-0.6,3.4,5-5.8,4.4-3.6-8.8,7.4-10.2,13.6l10.4-8.2,3.6-3s3.6,2.2,3.8,0.6,4.8-7.4,6-7.2,3.2-2.6,3,0,7.4,8,7.4,8,3.2-1.8,4.6-0.4,5.6-19.8,5.6-19.8l25-10.6,43.6-3.4-16.999-6.8-61.001-11.4z");
        getChildren().add(pg442);
        SVGPath pg446 = new SVGPath();
        pg446.setStrokeWidth(2.0);
        pg446.setStroke(Color.web("#4c0000"));
        pg446.setContent("m51.4,85s-15-16.8-23.4-19.4c0,0-13.4-6.8-38,1");
        getChildren().add(pg446);
        SVGPath pg450 = new SVGPath();
        pg450.setStrokeWidth(2.0);
        pg450.setStroke(Color.web("#4c0000"));
        pg450.setContent("m24.8,64.2s-25.2-8-40.6-3.8c0,0-18.4,2-26.8,15.8");
        getChildren().add(pg450);
        SVGPath pg454 = new SVGPath();
        pg454.setStrokeWidth(2.0);
        pg454.setStroke(Color.web("#4c0000"));
        pg454.setContent("m21.2,63s-17-7.2-31.8-9.4c0,0-16.6-2.6-33.2,4.6,0,0-12.2,6-17.6,16.2");
        getChildren().add(pg454);
        SVGPath pg458 = new SVGPath();
        pg458.setStrokeWidth(2.0);
        pg458.setStroke(Color.web("#4c0000"));
        pg458.setContent("m22.2,63.4s-15.4-11-16.4-12.4c0,0-7-11-20-11.4,0,0-21.4,0.8-38.6,8.8");
        getChildren().add(pg458);
        SVGPath pg462 = new SVGPath();
        pg462.setFill(Color.BLACK);
        pg462.setContent("M20.895,54.407c1.542,1.463,28.505,30.393,28.505,30.393,35.2,36.6,7.2,2.4,7.2,2.4-7.6-4.8-16.8-23.6-16.8-23.6-1.2-2.8,14,7.2,14,7.2,4,0.8,17.6,20,17.6,20-6.8-2.4-2,4.8-2,4.8,2.8,2,23.201,17.6,23.201,17.6,3.6,4,7.599,5.6,7.599,5.6,14-5.2,7.6,8,7.6,8,2.4,6.8,8-4.8,8-4.8,11.2-16.8-5.2-14.4-5.2-14.4-30,2.8-36.8-13.2-36.8-13.2-2.4-2.4,6.4,0,6.4,0,8.401,2-7.2-12.4-7.2-12.4,2.4,0,11.6,6.8,11.6,6.8,10.401,9.2,12.401,7.2,12.401,7.2,17.999-8.8,28.399-1.2,28.399-1.2,2,1.6-3.6,8.4-2,13.6s6.4,17.6,6.4,17.6c-2.4,1.6-2,12.4-2,12.4,16.8,23.2,7.2,21.2,7.2,21.2-15.6-0.4-0.8,7.2-0.8,7.2,3.2,2,12,9.2,12,9.2-2.8-1.2-4.4,4-4.4,4,4.8,4,2,8.8,2,8.8-6,1.2-7.2,5.2-7.2,5.2,6.8,8-3.2,8.4-3.2,8.4,3.6,4.4-1.2,16.4-1.2,16.4-4.8,0-11.2,5.6-11.2,5.6,2.4,4.8-8,10.4-8,10.4-8.4,1.6-5.6,8.4-5.6,8.4-7.999,6-10.399,22-10.399,22-0.8,10.4-3.2,13.6,2,11.6,5.199-2,4.399-14.4,4.399-14.4-4.799-15.6,38-31.6,38-31.6,4-1.6,4.8-6.8,4.8-6.8,2,0.4,10.8,8,10.8,8,7.6,11.2,8,2,8,2,1.2-3.6-0.4-9.6-0.4-9.6,6-21.6-8-28-8-28-10-33.6,4-25.2,4-25.2,2.8,5.6,13.6,10.8,13.6,10.8l3.6-2.4c-1.6-4.8,6.8-10.8,6.8-10.8,2.8,6.4,8.8-1.6,8.8-1.6,3.6-24.4,16-10,16-10,4,1.2,5.2-5.6,5.2-5.6,3.6-10.4,0-24,0-24,3.6-0.4,13.2,5.6,13.2,5.6,2.8-3.6-6.4-20.4-2.4-18s8.4,4,8.4,4c0.8-2-9.2-14.4-9.2-14.4-4.4-2.8-9.6-23.2-9.6-23.2,7.2,3.6-2.8-11.6-2.8-11.6,0-3.2,6-14.4,6-14.4-0.8-6.8,0-6.4,0-6.4,2.8,1.2,10.8,2.8,4-3.6s0.8-11.2,0.8-11.2c4.4-2.8-9.2-2.4-9.2-2.4-5.2-4.4-4.8-8.4-4.8-8.4,8,2-6.4-12.4-8.8-16s7.2-8.8,7.2-8.8c13.2-3.6,1.6-6.8,1.6-6.8-19.6,0.4-8.8-10.4-8.8-10.4,6,0.4,4.4-2,4.4-2-5.2-1.2-14.8-7.6-14.8-7.6-4-3.6-0.4-2.8-0.4-2.8,16.8,1.2-12-10-12-10,8,0-10-10.4-10-10.4-2-1.6-5.2-9.2-5.2-9.2-6-5.2-10.8-12-10.8-12-0.4-4.4-5.2-9.2-5.2-9.2-11.6-13.6-17.2-13.2-17.2-13.2-14.8-3.6-20-2.8-20-2.8l-52.8,4.4c-26.4,12.8-18.6,33.8-18.6,33.8,6.4,8.4,15.6,4.6,15.6,4.6,4.6-6.2,16.2-4,16.2-4,20.401,3.2,17.801-0.4,17.801-0.4-2.4-4.6-18.601-10.8-18.801-11.4s-9-4-9-4c-3-1.2-7.4-10.4-7.4-10.4-3.2-3.4,12.6,2.4,12.6,2.4-1.2,1,6.2,5,6.2,5,17.401-1,28.001,9.8,28.001,9.8,10.799,16.6,10.999,8.4,10.999,8.4,2.8-9.4-9-30.6-9-30.6,0.4-2,8.6,4.6,8.6,4.6,1.4-2,2.2,3.8,2.2,3.8,0.2,2.4,4,10.4,4,10.4,2.8,13,6.4,5.6,6.4,5.6l4.6,9.4c1.4,2.6-4.6,10.2-4.6,10.2-0.2,2.8,0.6,2.6-5,10.2s-2.2,12-2.2,12c-1.4,6.6,7.4,6.2,7.4,6.2,2.6,2.2,6,2.2,6,2.2,1.8,2,4.2,1.4,4.2,1.4,1.6-3.8,7.8-1.8,7.8-1.8,1.4-2.4,9.6-2.8,9.6-2.8,1-2.6,1.4-4.2,4.8-4.8s-21.2-43.6-21.2-43.6c6.4-0.8-1.8-13.2-1.8-13.2-2.2-6.6,9.2,8,11.4,9.4s3.2,3.6,1.6,3.4-3.4,2-2,2.2,14.4,15.2,17.8,25.4,9.4,14.2,15.6,20.2,5.4,30.2,5.4,30.2c-0.4,8.8,5.6,19.4,5.6,19.4,2,3.8-2.2,22-2.2,22-2,2.2-0.6,3-0.6,3,1,1.2,7.8,14.4,7.8,14.4-1.8-0.2,1.8,3.4,1.8,3.4,5.2,6-1.2,3-1.2,3-6-1.6,1,8.2,1,8.2,1.2,1.8-7.8-2.8-7.8-2.8-9.2-0.6,2.4,6.6,2.4,6.6,8.6,7.2-2.8,2.8-2.8,2.8-4.6-1.8-1.4,5-1.4,5,3.2,1.6,20.4,8.6,20.4,8.6,0.4,3.8-2.6,8.8-2.6,8.8,0.4,4-1.8,7.4-1.8,7.4-1.2,8.2-1.8,9-1.8,9-4.2,0.2-11.6,14-11.6,14-1.8,2.6-12,14.6-12,14.6-2,7-20-0.2-20-0.2-6.6,3.4-4.6,0-4.6,0-0.4-2.2,4.4-8.2,4.4-8.2,7-2.6,4.4-13.4,4.4-13.4,4-1.4-7.2-4.2-7-5.4s6-2.6,6-2.6c8-2,3.6-4.4,3.6-4.4-0.6-4,2.4-9.6,2.4-9.6,11.6-0.8,0-17,0-17-10.8-7.6-11.8-13.4-11.8-13.4,12.6-8.2,4.4-20.6,4.6-24.2s1.4-25.2,1.4-25.2c-2-6.2-5-19.8-5-19.8,2.2-5.2,9.6-17.8,9.6-17.8,2.8-4.2,11.6-9,9.4-12s-10-1.2-10-1.2c-7.8-1.4-7.2,3.8-7.2,3.8-1.6,1-2.4,6-2.4,6-0.72,7.933-9.6,14.2-9.6,14.2-11.2,6.2-2,10.2-2,10.2,6,6.6-3.8,6.8-3.8,6.8-11-1.8-2.8,8.4-2.8,8.4,10.8,12.8,7.8,15.6,7.8,15.6-10.2,1,2.4,10.2,2.4,10.2s-0.8-2-0.6-0.2,3.2,6,4,8-3.2,2.2-3.2,2.2c0.6,9.6-14.8,5.4-14.8,5.4l-1.6,0.2c-1.6,0.2-12.8-0.6-18.6-2.8s-12.599-2.2-12.599-2.2-4,1.8-11.601,1.6c-7.6-0.2-15.6,2.6-15.6,2.6-4.4-0.4,4.2-4.8,4.4-4.6s5.8-5.4-2.2-4.8c-21.797,1.635-32.6-8.6-32.6-8.6-2-1.4-4.6-4.2-4.6-4.2-10-2,1.4,12.4,1.4,12.4,1.2,1.4-0.2,2.4-0.2,2.4-0.8-1.6-8.6-7-8.6-7-2.811-0.973-4.174-2.307-6.505-4.793z");
        getChildren().add(pg462);
        SVGPath pg466 = new SVGPath();
        pg466.setFill(Color.web("#4c0000"));
        pg466.setContent("m-3,42.8s11.6,5.6,14.2,8.4,16.6,14.2,16.6,14.2-5.4-2-8-3.8-13.4-10-13.4-10-3.8-6-9.4-8.8z");
        getChildren().add(pg466);
        SVGPath pg470 = new SVGPath();
        pg470.setFill(Color.web("#99cc32"));
        pg470.setContent("M-61.009,11.603c0.337-0.148-0.187-2.86-0.391-3.403-1.022-2.726-10-4.2-10-4.2-0.227,1.365-0.282,2.961-0.176,4.599,0,0,4.868,5.519,10.567,3.004z");
        getChildren().add(pg470);
        SVGPath pg474 = new SVGPath();
        pg474.setFill(Color.web("#659900"));
        pg474.setContent("M-61.009,11.403c-0.449,0.158-0.015-2.734-0.191-3.203-1.022-2.726-10.2-4.3-10.2-4.3-0.227,1.365-0.282,2.961-0.176,4.599,0,0,4.268,5.119,10.567,2.904z");
        getChildren().add(pg474);
        SVGPath pg478 = new SVGPath();
        pg478.setFill(Color.BLACK);
        pg478.setContent("m-65.4,11.546c-0.625,0-1.131-1.14-1.131-2.546,0-1.405,0.506-2.545,1.131-2.545s1.132,1.14,1.132,2.545c0,1.406-0.507,2.546-1.132,2.546z");
        getChildren().add(pg478);
        SVGPath pg482 = new SVGPath();
        pg482.setFill(Color.BLACK);
        pg482.setContent("M-65.4,9z");
        getChildren().add(pg482);
        SVGPath pg486 = new SVGPath();
        pg486.setFill(Color.BLACK);
        pg486.setContent("m-111,109.6s-5.6,10,19.2,4c0,0,14-1.2,16.4-3.6,1.2,0.8,9.566,3.73,12.4,4.4,6.8,1.6,15.2-8.4,15.2-8.4s4.6-10.5,7.4-10.5-0.4,1.6-0.4,1.6-6.6,10.1-6.2,11.7c0,0-5.2,20-21.2,20.8,0,0-16.15,0.95-14.8,6.8,0,0,8.8-2.4,11.2,0,0,0,10.8-0.4,2.8,6l-6.8,11.6s0.14,3.92-10,0.4c-9.8-3.4-20.1-16.3-20.1-16.3s-15.95-14.55-5.1-28.5z");
        getChildren().add(pg486);
        SVGPath pg490 = new SVGPath();
        pg490.setFill(Color.web("#e59999"));
        pg490.setContent("m-112.2,113.6s-2,9.6,34.8-0.8l6.8,0.8c2.4,0.8,14.4,3.6,16.4,2.4,0,0-7.2,13.6-18.8,12,0,0-13.2,1.6-12.8,6.4,0,0,4,7.2,8.8,9.6,0,0,2.8,2.4,2.4,5.6s-3.2,4.8-5.2,5.6-5.2-2.4-6.8-2.4-10-6.4-14.4-11.2-12.8-16.8-12.4-19.6,1.2-8.4,1.2-8.4z");
        getChildren().add(pg490);
        SVGPath pg494 = new SVGPath();
        pg494.setFill(Color.web("#b26565"));
        pg494.setContent("m-109,131.05c2.6,3.95,5.8,8.15,8,10.55,4.4,4.8,12.8,11.2,14.4,11.2s4.8,3.2,6.8,2.4,4.8-2.4,5.2-5.6-2.4-5.6-2.4-5.6c-3.066-1.53-5.806-5.02-7.385-7.35,0,0,0.185,2.55-5.015,1.75s-10.4-3.6-12-6.8-4-5.6-2.4-2,4,7.2,5.6,7.6,1.2,1.6-1.2,1.2-5.2-0.8-9.6-6z");
        getChildren().add(pg494);
        SVGPath pg498 = new SVGPath();
        pg498.setFill(Color.web("#992600"));
        pg498.setContent("m-111.6,110s1.8-13.6,3-17.6c0,0-0.8-6.8,1.6-11s4.4-10.4,7.4-15.8,3.2-9.4,7.2-11,10-10.2,12.8-11.2,2.6-0.2,2.6-0.2,6.8-14.8,20.4-10.8c0,0-16.2-2.8-0.4-12.2,0,0-4.8,1.1-1.5-5.9,2.201-4.668,1.7,2.1-9.3,13.9,0,0-5,8.6-10.2,11.6s-17.2,10-18.4,13.8-4.4,9.6-6.4,11.2-4.8,5.8-5.2,9.2c0,0-1.2,4-2.6,5.2s-1.6,4.4-1.6,6.4-2,4.8-1.8,7.2c0,0,0.8,19,0.4,21l2-3.8z");
        getChildren().add(pg498);
        SVGPath pg502 = new SVGPath();
        pg502.setFill(Color.WHITE);
        pg502.setContent("m-120.2,114.6s-2-1.4-6.4,4.6c0,0,7.3,33,7.3,34.4,0,0,1.1-2.1-0.2-9.3s-2.2-19.9-2.2-19.9l1.5-9.8z");
        getChildren().add(pg502);
        SVGPath pg506 = new SVGPath();
        pg506.setFill(Color.web("#992600"));
        pg506.setContent("m-98.6,54s-17.6,3.2-17.2,32.4l-0.8,24.8s-1.2-25.6-2.4-27.2,2.8-12.8-0.4-6.8c0,0-14,14-6,35.2,0,0,1.5,3.3-1.5-1.3,0,0-4.6-12.6-3.5-19,0,0,0.2-2.2,2.1-5,0,0,8.6-11.7,11.3-14,0,0,1.8-14.4,17.2-19.6,0,0,5.7-2.3,1.2,0.5z");
        getChildren().add(pg506);
        SVGPath pg510 = new SVGPath();
        pg510.setFill(Color.BLACK);
        pg510.setContent("m40.8-12.2c0.66-0.354,0.651-1.324,1.231-1.497,1.149-0.344,1.313-1.411,1.831-2.195,0.873-1.319,1.066-2.852,1.648-4.343,0.272-0.7,0.299-1.655-0.014-2.315-1.174-2.481-1.876-4.93-3.318-7.356-0.268-0.45-0.53-1.244-0.731-1.842-0.463-1.384-1.72-2.375-2.58-3.695-0.288-0.441,0.237-1.366-0.479-1.45-0.897-0.105-2.346-0.685-2.579,0.341-0.588,2.587,0.423,5.11,1.391,7.552-0.782,0.692-0.448,1.613-0.296,2.38,0.71,3.606-0.488,6.958-1.249,10.432-0.023,0.104,0.319,0.302,0.291,0.364-1.222,2.686-2.674,5.131-4.493,7.512-0.758,0.992-1.63,1.908-2.127,2.971-0.368,0.787-0.776,1.753-0.526,2.741-3.435,2.78-5.685,6.625-8.296,10.471-0.462,0.68-0.171,1.889,0.38,2.158,0.813,0.398,1.769-0.626,2.239-1.472,0.389-0.698,0.742-1.348,1.233-1.991,0.133-0.175-0.046-0.594,0.089-0.715,2.633-2.347,4.302-5.283,6.755-7.651,1.95-0.329,3.487-1.327,5.235-2.34,0.308-0.179,0.832,0.07,1.122-0.125,1.753-1.177,1.751-3.213,1.857-5.123,0.05-0.884,0.246-2.201,1.386-2.812z");
        getChildren().add(pg510);
        SVGPath pg514 = new SVGPath();
        pg514.setFill(Color.BLACK);
        pg514.setContent("m31.959-16.666c0.124-0.077-0.031-0.5,0.078-0.716,0.162-0.324,0.565-0.512,0.727-0.836,0.109-0.216-0.054-0.596,0.082-0.738,2.333-2.447,2.59-5.471,1.554-8.444,1.024-0.62,1.085-1.882,0.66-2.729-0.853-1.7-1.046-3.626-2.021-5.169-0.802-1.269-2.38-2.513-3.751-1.21-0.421,0.4-0.742,1.187-0.464,1.899,0.064,0.163,0.349,0.309,0.322,0.391-0.107,0.324-0.653,0.548-0.659,0.82-0.03,1.496-0.984,3.007-0.354,4.336,0.772,1.629,1.591,3.486,2.267,5.262-1.234,2.116-0.201,4.565-1.954,6.442-0.136,0.146-0.127,0.532-0.005,0.734,0.292,0.486,0.698,0.892,1.184,1.184,0.202,0.121,0.55,0.123,0.75-0.001,0.578-0.362,0.976-0.849,1.584-1.225z");
        getChildren().add(pg514);
        SVGPath pg518 = new SVGPath();
        pg518.setFill(Color.BLACK);
        pg518.setContent("m94.771-26.977c1.389,1.792,1.679,4.587-0.37,5.977,0.55,3.309,3.901,1.33,5.999,0.8-0.11-0.388,0.12-0.732,0.4-0.737,1.06-0.015,1.74-1.047,2.8-0.863,0.44-1.557,2.07-2.259,2.72-3.639,1.72-3.695,1.13-7.968-1.45-11.214-0.2-0.254,0.01-0.771-0.11-1.133-0.76-2.211-2.82-2.526-4.76-3.214-1.176-3.875-1.837-7.906-3.599-11.6-1.614-0.25-2.312-1.989-3.649-2.709-1.333-0.719-1.901,0.86-1.86,1.906,0.007,0.205,0.459,0.429,0.289,0.794-0.076,0.164-0.336,0.275-0.336,0.409,0.001,0.135,0.222,0.266,0.356,0.4-0.918,0.82-2.341,1.297-2.636,2.442-0.954,3.71,1.619,6.835,3.287,10.036,0.591,1.135-0.145,2.406-0.905,3.614-0.438,0.695-0.33,1.822-0.054,2.678,0.752,2.331,2.343,4.07,3.878,6.053z");
        getChildren().add(pg518);
        SVGPath pg522 = new SVGPath();
        pg522.setFill(Color.BLACK);
        pg522.setContent("m57.611-8.591c-1.487,1.851-4.899,4.42-1.982,6.348,0.194,0.129,0.564,0.133,0.737-0.001,2.021-1.565,4.024-2.468,6.46-3.05,0.124-0.029,0.398,0.438,0.767,0.277,1.613-0.703,3.623-0.645,4.807-1.983,3.767,0.224,7.332-0.892,10.723-2.2,1.161-0.448,2.431-1.007,3.632-1.509,1.376-0.576,2.58-1.504,3.692-2.645,0.133-0.136,0.487-0.046,0.754-0.046-0.04-0.863,0.922-0.99,1.169-1.612,0.092-0.232-0.058-0.628,0.075-0.73,2.138-1.63,3.058-3.648,1.889-6.025-0.285-0.578-0.534-1.196-1.1-1.672-1.085-0.911-2.187-0.057-3.234-0.361-0.159,0.628-0.888,0.456-1.274,0.654-0.859,0.439-2.192-0.146-3.051,0.292-1.362,0.695-2.603,0.864-4.025,1.241-0.312,0.082-1.09-0.014-1.25,0.613-0.134-0.134-0.282-0.368-0.388-0.346-1.908,0.396-3.168,0.61-4.469,2.302-0.103,0.133-0.545-0.046-0.704,0.089-0.957,0.808-1.362,2.042-2.463,2.714-0.201,0.123-0.553-0.045-0.747,0.084-0.646,0.431-1.013,1.072-1.655,1.519-0.329,0.229-0.729-0.096-0.697-0.352,0.245-1.947,0.898-3.734,0.323-5.61,2.077-2.52,4.594-4.469,6.4-7.2,0.015-2.166,0.707-4.312,0.594-6.389-0.01-0.193-0.298-0.926-0.424-1.273-0.312-0.854,0.594-1.92-0.25-2.644-1.404-1.203-2.696-0.327-3.52,1.106-1.838,0.39-3.904,1.083-5.482-0.151-1.007-0.787-1.585-1.693-2.384-2.749-0.985-1.302-0.65-2.738-0.58-4.302,0.006-0.128-0.309-0.264-0.309-0.398,0.001-0.135,0.221-0.266,0.355-0.4-0.706-0.626-0.981-1.684-2-2,0.305-1.092-0.371-1.976-1.242-2.278-1.995-0.691-3.672,1.221-5.564,1.294-0.514,0.019-0.981-1.019-1.63-1.344-0.432-0.216-1.136-0.249-1.498,0.017-0.688,0.504-1.277,0.618-2.035,0.823-1.617,0.436-2.895,1.53-4.375,2.385-1.485,0.857-2.44,2.294-3.52,3.614-0.941,1.152-1.077,3.566,0.343,4.066,1.843,0.65,3.147-2.053,5.113-1.727,0.312,0.051,0.518,0.362,0.408,0.75,0.389,0.109,0.607-0.12,0.8-0.4,0.858,1.019,2.022,1.356,2.96,2.229,0.97,0.904,2.716,0.486,3.731,1.483,1.529,1.502,0.97,4.183,2.909,5.488-0.586,1.313-1.193,2.59-1.528,4.017-0.282,1.206,0.712,2.403,1.923,2.312,1.258-0.094,1.52-0.853,2.005-1.929,0.267,0.267,0.736,0.564,0.695,0.78-0.457,2.387-1.484,4.38-1.942,6.811-0.059,0.317-0.364,0.519-0.753,0.409-0.468,4.149-4.52,6.543-7.065,9.708-0.403,0.502-0.407,1.751,0.002,2.154,1.403,1.387,3.363-0.159,5.063-0.662,0.213-1.206,1.072-2.148,2.404-2.092,0.256,0.01,0.491-0.532,0.815-0.662,0.348-0.138,0.85,0.086,1.136-0.112,1.729-1.195,3.137-2.301,4.875-3.49,0.192-0.131,0.536,0.028,0.752-0.08,0.325-0.162,0.512-0.549,0.835-0.734,0.348-0.2,0.59,0.09,0.783,0.37-0.646,0.349-0.65,1.306-1.232,1.508-0.775,0.268-1.336,0.781-2.01,1.228-0.292,0.193-0.951-0.055-1.055,0.124-0.598,1.028-1.782,1.466-2.492,2.349z");
        getChildren().add(pg522);
        SVGPath pg526 = new SVGPath();
        pg526.setFill(Color.BLACK);
        pg526.setContent("m2.2-58s-9.238-2.872-20.4,22.8c0,0-2.4,5.2-4.8,7.2s-13.6,5.6-15.6,9.6l-10.4,16s14.8-16,18-18.4c0,0,8-8.4,4.8-1.6,0,0-14,10.8-12.8,20,0,0-5.6,14.4-6.4,16.4,0,0,16-32,18.4-33.2s3.6-1.2,2.4,2.4-1.6,20-4.4,22c0,0,8-20.4,7.2-23.6,0,0,3.2-3.6,5.6,1.6l-1.2,16,4.4,12s-2.4-11.2-0.8-26.8c0,0-2-10.4,2-4.8s13.6,11.6,13.6,16.4c0,0-5.2-17.6-14.4-22.4l-4,6-1.2-2s-3.6-0.8,0.8-7.6,4-7.6,4-7.6,6.4,7.2,8,7.2c0,0,13.2-7.6,14.4,16.8,0,0,6.8-14.4-2.4-21.2,0,0-14.8-2-13.6-7.2l7.2-12.4c3.6-5.2,2-2.4,2-2.4z");
        getChildren().add(pg526);
        SVGPath pg530 = new SVGPath();
        pg530.setFill(Color.BLACK);
        pg530.setContent("m-17.8-41.6-16,5.2-7.2,9.6s17.2-10,21.2-11.2,2-3.6,2-3.6z");
        getChildren().add(pg530);
        SVGPath pg534 = new SVGPath();
        pg534.setFill(Color.BLACK);
        pg534.setContent("m-57.8-35.2s-2,1.2-2.4,4-2.8,3.2-2,6,2.8,5.2,2.8,1.2,1.6-6,2.4-7.2,2.4-5.6-0.8-4z");
        getChildren().add(pg534);
        SVGPath pg538 = new SVGPath();
        pg538.setFill(Color.BLACK);
        pg538.setContent("m-66.6,26s-8.4-4-11.6-7.6-2.748,1.566-7.6,1.2c-5.847-0.441-4.8-16.4-4.8-16.4l-4,7.6s-1.2,14.4,6.8,12c3.907-1.172,5.2,0.4,3.6,1.2s5.6,1.2,2.8,2.8,11.6-3.6,9.2,6.8l5.6-7.6z");
        getChildren().add(pg538);
        SVGPath pg542 = new SVGPath();
        pg542.setFill(Color.BLACK);
        pg542.setContent("m-79.2,40.4s-15.4,4.4-19-5.2c0,0-4.8,2.4-2.6,5.4s3.4,3.4,3.4,3.4,5.4,1.2,4.8,2-3,4.2-3,4.2,10.2-6,16.4-9.8z");
        getChildren().add(pg542);
        SVGPath pg546 = new SVGPath();
        pg546.setFill(Color.WHITE);
        pg546.setContent("m149.2,118.6c-0.43,2.14-2.1,2.94-4,3.6-1.92-0.96-4.51-4.06-6.4-2-0.47-0.48-1.25-0.54-1.6-1.2-0.46-0.9-0.19-1.94-0.53-2.74-0.55-1.28-1.25-2.64-1.07-4.06,1.81-0.71,2.4-2.62,1.93-4.38-0.07-0.26-0.5-0.45-0.3-0.8,0.19-0.33,0.5-0.55,0.77-0.82-0.13,0.14-0.28,0.37-0.39,0.35-0.61-0.11-0.49-0.75-0.36-1.13,0.59-1.75,2.6-2.01,3.95-0.82,0.26-0.56,0.77-0.37,1.2-0.4-0.05-0.58,0.36-1.11,0.56-1.53,0.52-1.09,2.14,0.01,2.94-0.6,1.08-0.83,2.14-1.52,3.22-0.92,1.81,1.01,3.52,2.22,4.72,3.97,0.57,0.83,0.81,2.11,0.75,3.07-0.04,0.65-1.42,0.29-1.76,1.22-0.65,1.75,1.19,2.27,1.94,3.61,0.2,0.35-0.06,0.65-0.38,0.75-0.41,0.13-1.19-0.06-1.06,0.39,0.98,3.19-1.78,3.87-4.13,4.44z");
        getChildren().add(pg546);
        SVGPath pg550 = new SVGPath();
        pg550.setFill(Color.WHITE);
        pg550.setContent("m139.6,138.2c-0.01-1.74-1.61-3.49-0.4-5.2,0.14,0.14,0.27,0.36,0.4,0.36,0.14,0,0.27-0.22,0.4-0.36,1.5,2.22,5.15,3.14,5.01,5.99-0.03,0.45-1.11,1.37-0.21,2.01-1.81,1.35-1.87,3.72-2.8,5.6-1.24-0.28-2.45-0.65-3.6-1.2,0.35-1.48,0.24-3.17,1.06-4.49,0.43-0.7,0.14-1.78,0.14-2.71z");
        getChildren().add(pg550);
        SVGPath pg554 = new SVGPath();
        pg554.setFill(Color.web("#CCC"));
        pg554.setContent("m-26.6,129.2s-16.858,10.14-2.8-5.2c8.8-9.6,18.8-15.2,18.8-15.2s10.4-4.4,14-5.6,18.8-6.4,22-6.8,12.8-4.4,19.6-0.4,14.8,8.4,14.8,8.4-16.4-8.4-20-6-10.8,2-16.8,5.2c0,0-14.8,4.4-18,6.4s-13.6,13.6-15.2,12.8,0.4-1.2,1.6-4-0.8-4.4-8.8,2-9.2,8.4-9.2,8.4z");
        getChildren().add(pg554);
        SVGPath pg558 = new SVGPath();
        pg558.setFill(Color.BLACK);
        pg558.setContent("m-19.195,123.23s1.41-13.04,9.888-11.37c0,0,8.226-4.17,10.948-6.14,0,0,8.139-1.7,9.449-2.32,18.479-8.698,33.198-4.179,33.745-5.299,0.546-1.119,20.171,5.999,23.78,10.079,0.391,0.45-10.231-5.59-19.929-7.48-8.273-1.617-29.875,0.24-40.781,5.78-2.973,1.51-11.918,7.29-14.449,7.18s-12.651,9.57-12.651,9.57z");
        getChildren().add(pg558);
        SVGPath pg562 = new SVGPath();
        pg562.setFill(Color.web("#CCC"));
        pg562.setContent("m-23,148.8s-15.2-2.4,1.6-4c0,0,18-2,22-7.2,0,0,13.6-9.2,16.4-9.6s32.8-7.6,33.2-10,6-2.4,7.6-1.6,0.8,2-2,2.8-34,17.2-40.4,18.4-18,8.8-22.8,10-15.6,1.2-15.6,1.2z");
        getChildren().add(pg562);
        SVGPath pg566 = new SVGPath();
        pg566.setFill(Color.BLACK);
        pg566.setContent("m-3.48,141.4s-8.582-0.83,0.019-1.64c0,0,8.816-3.43,10.864-6.09,0,0,6.964-4.71,8.397-4.92,1.434-0.2,15.394-3.89,15.599-5.12s34.271-13.81,38.691-10.62c2.911,2.1-6.99,0.43-16.624,4.84-1.355,0.62-35.208,15.2-38.485,15.82-3.277,0.61-9.216,4.5-11.674,5.12-2.457,0.61-6.787,2.61-6.787,2.61z");
        getChildren().add(pg566);
        SVGPath pg570 = new SVGPath();
        pg570.setFill(Color.BLACK);
        pg570.setContent("m-11.4,143.6s5.2-0.4,4,1.2-3.6,0.8-3.6,0.8l-0.4-2z");
        getChildren().add(pg570);
        SVGPath pg574 = new SVGPath();
        pg574.setFill(Color.BLACK);
        pg574.setContent("m-18.6,145.2s5.2-0.4,4,1.2-3.6,0.8-3.6,0.8l-0.4-2z");
        getChildren().add(pg574);
        SVGPath pg578 = new SVGPath();
        pg578.setFill(Color.BLACK);
        pg578.setContent("m-29,146.8s5.2-0.4,4,1.2-3.6,0.8-3.6,0.8l-0.4-2z");
        getChildren().add(pg578);
        SVGPath pg582 = new SVGPath();
        pg582.setFill(Color.BLACK);
        pg582.setContent("m-36.6,147.6s5.2-0.4,4,1.2-3.6,0.8-3.6,0.8l-0.4-2z");
        getChildren().add(pg582);
        SVGPath pg586 = new SVGPath();
        pg586.setFill(Color.BLACK);
        pg586.setContent("m1.8,108,3.2,1.6c-1.2,1.6-4.4,1.2-4.4,1.2l1.2-2.8z");
        getChildren().add(pg586);
        SVGPath pg590 = new SVGPath();
        pg590.setFill(Color.BLACK);
        pg590.setContent("m-8.2,113.6s6.506-2.14,4,1.2c-1.2,1.6-3.6,0.8-3.6,0.8l-0.4-2z");
        getChildren().add(pg590);
        SVGPath pg594 = new SVGPath();
        pg594.setFill(Color.BLACK);
        pg594.setContent("m-19.4,118.4s5.2-0.4,4,1.2-3.6,0.8-3.6,0.8l-0.4-2z");
        getChildren().add(pg594);
        SVGPath pg598 = new SVGPath();
        pg598.setFill(Color.BLACK);
        pg598.setContent("m-27,124.4s5.2-0.4,4,1.2-3.6,0.8-3.6,0.8l-0.4-2z");
        getChildren().add(pg598);
        SVGPath pg602 = new SVGPath();
        pg602.setFill(Color.BLACK);
        pg602.setContent("m-33.8,129.2s5.2-0.4,4,1.2-3.6,0.8-3.6,0.8l-0.4-2z");
        getChildren().add(pg602);
        SVGPath pg606 = new SVGPath();
        pg606.setFill(Color.BLACK);
        pg606.setContent("m5.282,135.6s6.921-0.53,5.324,1.6c-1.597,2.12-4.792,1.06-4.792,1.06l-0.532-2.66z");
        getChildren().add(pg606);
        SVGPath pg610 = new SVGPath();
        pg610.setFill(Color.BLACK);
        pg610.setContent("m15.682,130.8s6.921-0.53,5.324,1.6c-1.597,2.12-4.792,1.06-4.792,1.06l-0.532-2.66z");
        getChildren().add(pg610);
        SVGPath pg614 = new SVGPath();
        pg614.setFill(Color.BLACK);
        pg614.setContent("m26.482,126.4s6.921-0.53,5.324,1.6c-1.597,2.12-4.792,1.06-4.792,1.06l-0.532-2.66z");
        getChildren().add(pg614);
        SVGPath pg618 = new SVGPath();
        pg618.setFill(Color.BLACK);
        pg618.setContent("m36.882,121.6s6.921-0.53,5.324,1.6c-1.597,2.12-4.792,1.06-4.792,1.06l-0.532-2.66z");
        getChildren().add(pg618);
        SVGPath pg622 = new SVGPath();
        pg622.setFill(Color.BLACK);
        pg622.setContent("m9.282,103.6s6.921-0.53,5.324,1.6c-1.597,2.12-5.592,1.86-5.592,1.86l0.268-3.46z");
        getChildren().add(pg622);
        SVGPath pg626 = new SVGPath();
        pg626.setFill(Color.BLACK);
        pg626.setContent("m19.282,100.4s6.921-0.534,5.324,1.6c-1.597,2.12-5.992,1.86-5.992,1.86l0.668-3.46z");
        getChildren().add(pg626);
        SVGPath pg630 = new SVGPath();
        pg630.setFill(Color.BLACK);
        pg630.setContent("m-3.4,140.4s5.2-0.4,4,1.2-3.6,0.8-3.6,0.8l-0.4-2z");
        getChildren().add(pg630);
        SVGPath pg634 = new SVGPath();
        pg634.setFill(Color.web("#992600"));
        pg634.setContent("m-76.6,41.2s-4.4,8.8-4.8,12c0,0,0.8-8.8,2-10.8s2.8-1.2,2.8-1.2z");
        getChildren().add(pg634);
        SVGPath pg638 = new SVGPath();
        pg638.setFill(Color.web("#992600"));
        pg638.setContent("m-95,55.2s-3.2,14.4-2.8,17.2c0,0-1.2-11.6-0.8-12.8s3.6-4.4,3.6-4.4z");
        getChildren().add(pg638);
        SVGPath pg642 = new SVGPath();
        pg642.setFill(Color.web("#CCC"));
        pg642.setContent("m-74.2-19.4-0.2,3.2-2.2,0.2s14.2,12.6,14.8,20.2c0,0,0.8-8.2-12.4-23.6z");
        getChildren().add(pg642);
        SVGPath pg646 = new SVGPath();
        pg646.setFill(Color.BLACK);
        pg646.setContent("m-70.216-18.135c-0.431-0.416-0.212-1.161-0.62-1.421-0.809-0.516,1.298-0.573,1.07-1.289-0.383-1.206-0.196-1.227-0.318-2.503-0.057-0.598,0.531-2.138,0.916-2.578,1.446-1.652,0.122-4.584,1.762-6.135,0.304-0.289,0.68-0.841,0.965-1.259,0.659-0.963,1.843-1.451,2.793-2.279,0.318-0.276,0.117-1.103,0.686-1.011,0.714,0.115,1.955-0.015,1.91,0.826-0.113,2.12-1.442,3.84-2.722,5.508,0.451,0.704-0.007,1.339-0.291,1.896-1.335,2.62-1.146,5.461-1.32,8.301-0.005,0.085-0.312,0.163-0.304,0.216,0.353,2.335,0.937,4.534,1.816,6.763,0.366,0.93,0.837,1.825,0.987,2.752,0.111,0.686,0.214,1.519-0.194,2.224,2.035,2.89,0.726,5.541,1.895,9.072,0.207,0.625,1.899,2.539,1.436,2.378-2.513-0.871-2.625-1.269-2.802-2.022-0.146-0.623-0.476-2-0.713-2.602-0.064-0.164-0.235-2.048-0.313-2.17-1.513-2.382-0.155-2.206-1.525-4.564-1.428-0.68-2.394-1.784-3.517-2.946-0.198-0.204,0.945-0.928,0.764-1.141-1.092-1.289-2.245-2.056-1.909-3.549,0.155-0.69,0.292-1.747-0.452-2.467z");
        getChildren().add(pg646);
        SVGPath pg650 = new SVGPath();
        pg650.setFill(Color.BLACK);
        pg650.setContent("m-73.8-16.4s0.4,6.8,2.8,8.4,1.2,0.8-2-0.4-2-2-2-2-2.8,0.4-0.4,2.4,6,4.4,4.4,4.4-9.2-4-9.2-6.8-1-6.9-1-6.9,1.1-0.8,5.9-0.7c0,0,1.4,0.7,1.5,1.6z");
        getChildren().add(pg650);
        SVGPath pg654 = new SVGPath();
        pg654.setStrokeWidth(0.1);
        pg654.setStroke(Color.web("#000"));
        pg654.setFill(Color.WHITE);
        pg654.setContent("m-74.6,2.2s-8.52-2.791-27,0.6c0,0,9.031-2.078,27.8,0.2,10.3,1.25-0.8-0.8-0.8-0.8z");
        getChildren().add(pg654);
        SVGPath pg658 = new SVGPath();
        pg658.setStrokeWidth(0.1);
        pg658.setStroke(Color.web("#000"));
        pg658.setFill(Color.WHITE);
        pg658.setContent("m-72.502,2.129s-8.246-3.518-26.951-1.737c0,0,9.178-1.289,27.679,2.603,10.154,2.136-0.728-0.866-0.728-0.866z");
        getChildren().add(pg658);
        SVGPath pg662 = new SVGPath();
        pg662.setStrokeWidth(0.1);
        pg662.setStroke(Color.web("#000"));
        pg662.setFill(Color.WHITE);
        pg662.setContent("m-70.714,2.222s-7.962-4.121-26.747-3.736c0,0,9.248-0.604,27.409,4.654,9.966,2.885-0.662-0.918-0.662-0.918z");
        getChildren().add(pg662);
        SVGPath pg666 = new SVGPath();
        pg666.setStrokeWidth(0.1);
        pg666.setStroke(Color.web("#000"));
        pg666.setFill(Color.WHITE);
        pg666.setContent("m-69.444,2.445s-6.824-4.307-23.698-5.405c0,0,8.339,0.17,24.22,6.279,8.716,3.353-0.522-0.874-0.522-0.874z");
        getChildren().add(pg666);
        SVGPath pg670 = new SVGPath();
        pg670.setStrokeWidth(0.1);
        pg670.setStroke(Color.web("#000"));
        pg670.setFill(Color.WHITE);
        pg670.setContent("m45.84,12.961s-0.93,0.644-0.716-0.537c0.215-1.181,28.423-14.351,32.037-14.101,0,0-30.248,13.206-31.321,14.638z");
        getChildren().add(pg670);
        SVGPath pg674 = new SVGPath();
        pg674.setStrokeWidth(0.1);
        pg674.setStroke(Color.web("#000"));
        pg674.setFill(Color.WHITE);
        pg674.setContent("m42.446,13.6s-0.876,0.715-0.755-0.479,27.208-16.539,30.83-16.573c0,0-29.117,15.541-30.075,17.052z");
        getChildren().add(pg674);
        SVGPath pg678 = new SVGPath();
        pg678.setStrokeWidth(0.1);
        pg678.setStroke(Color.web("#000"));
        pg678.setFill(Color.WHITE);
        pg678.setContent("m39.16,14.975s-0.828,0.772-0.786-0.428c0.042-1.199,19.859-16.696,29.671-18.57,0,0-18.03,8.127-28.885,18.998z");
        getChildren().add(pg678);
        SVGPath pg682 = new SVGPath();
        pg682.setStrokeWidth(0.1);
        pg682.setStroke(Color.web("#000"));
        pg682.setFill(Color.WHITE);
        pg682.setContent("m36.284,16.838s-0.745,0.694-0.707-0.385c0.038-1.08,17.872-15.027,26.703-16.713,0,0-16.226,7.314-25.996,17.098z");
        getChildren().add(pg682);
        SVGPath pg686 = new SVGPath();
        pg686.setFill(Color.web("#CCC"));
        pg686.setContent("m4.6,164.8s-15.2-2.4,1.6-4c0,0,18-2,22-7.2,0,0,13.6-9.2,16.4-9.6s19.2-4,19.6-6.4,6.4-4.8,8-4,1.6,10-1.2,10.8-21.6,8-28,9.2-18,8.8-22.8,10-15.6,1.2-15.6,1.2z");
        getChildren().add(pg686);
        SVGPath pg690 = new SVGPath();
        pg690.setFill(Color.BLACK);
        pg690.setContent("m77.6,127.4s-3,1.6-4.2,4.2c0,0-6.4,10.6-20.6,13.8,0,0-23,9-30.8,11,0,0-13.4,5-20.8,4.2,0,0-7,0.2-0.8,1.8,0,0,20.2-2,23.6-3.8,0,0,15.6-5.2,18.6-7.8s21.2-7.6,23.4-9.6,12-10.4,11.6-13.8z");
        getChildren().add(pg690);
        SVGPath pg694 = new SVGPath();
        pg694.setFill(Color.BLACK);
        pg694.setContent("m18.882,158.91s5.229-0.23,4.076,1.32-3.601,0.68-3.601,0.68l-0.475-2z");
        getChildren().add(pg694);
        SVGPath pg698 = new SVGPath();
        pg698.setFill(Color.BLACK);
        pg698.setContent("m11.68,160.26s5.228-0.22,4.076,1.33c-1.153,1.55-3.601,0.67-3.601,0.67l-0.475-2z");
        getChildren().add(pg698);
        SVGPath pg702 = new SVGPath();
        pg702.setFill(Color.BLACK);
        pg702.setContent("m1.251,161.51s5.229-0.23,4.076,1.32-3.601,0.68-3.601,0.68l-0.475-2z");
        getChildren().add(pg702);
        SVGPath pg706 = new SVGPath();
        pg706.setFill(Color.BLACK);
        pg706.setContent("m-6.383,162.06s5.229-0.23,4.076,1.32-3.601,0.67-3.601,0.67l-0.475-1.99z");
        getChildren().add(pg706);
        SVGPath pg710 = new SVGPath();
        pg710.setFill(Color.BLACK);
        pg710.setContent("m35.415,151.51s6.96-0.3,5.425,1.76c-1.534,2.07-4.793,0.9-4.793,0.9l-0.632-2.66z");
        getChildren().add(pg710);
        SVGPath pg714 = new SVGPath();
        pg714.setFill(Color.BLACK);
        pg714.setContent("m45.73,147.09s5.959-3.3,5.425,1.76c-0.27,2.55-4.793,0.9-4.793,0.9l-0.632-2.66z");
        getChildren().add(pg714);
        SVGPath pg718 = new SVGPath();
        pg718.setFill(Color.BLACK);
        pg718.setContent("m54.862,144.27s7.159-3.7,5.425,1.77c-0.778,2.44-4.794,0.9-4.794,0.9l-0.631-2.67z");
        getChildren().add(pg718);
        SVGPath pg722 = new SVGPath();
        pg722.setFill(Color.BLACK);
        pg722.setContent("m64.376,139.45s4.359-4.9,5.425,1.76c0.406,2.54-4.793,0.9-4.793,0.9l-0.632-2.66z");
        getChildren().add(pg722);
        SVGPath pg726 = new SVGPath();
        pg726.setFill(Color.BLACK);
        pg726.setContent("m26.834,156s5.228-0.23,4.076,1.32c-1.153,1.55-3.602,0.68-3.602,0.68l-0.474-2z");
        getChildren().add(pg726);
        SVGPath pg730 = new SVGPath();
        pg730.setStrokeWidth(0.1);
        pg730.setStroke(Color.web("#000"));
        pg730.setFill(Color.WHITE);
        pg730.setContent("m62.434,34.603s-0.726,0.665-0.727-0.406c0-1.07,17.484-14.334,26.327-15.718,0,0-16.099,6.729-25.6,16.124z");
        getChildren().add(pg730);
        SVGPath pg734 = new SVGPath();
        pg734.setFill(Color.BLACK);
        pg734.setContent("m65.4,98.4s22.001,22.4,31.201,26c0,0,9.199,11.2,5.199,37.2,0,0-3.199,7.6-6.399-13.2,0,0,3.2-25.2-8-9.2,0,0-8.401-9.9-2.001-9.6,0,0,3.201,2,3.601,0.4s-7.601-15.2-24.801-29.6,1.2-2,1.2-2z");
        getChildren().add(pg734);
        SVGPath pg738 = new SVGPath();
        pg738.setStrokeWidth(0.1);
        pg738.setStroke(Color.web("#000"));
        pg738.setFill(Color.WHITE);
        pg738.setContent("m7,137.2s-0.2-1.8,1.6-1,96,7,127.6,31c0,0-45.199-23.2-129.2-30z");
        getChildren().add(pg738);
        SVGPath pg742 = new SVGPath();
        pg742.setStrokeWidth(0.1);
        pg742.setStroke(Color.web("#000"));
        pg742.setFill(Color.WHITE);
        pg742.setContent("m17.4,132.8s-0.2-1.8,1.6-1,138.4-0.2,162,32.2c0,0-22-25.2-163.6-31.2z");
        getChildren().add(pg742);
        SVGPath pg746 = new SVGPath();
        pg746.setStrokeWidth(0.1);
        pg746.setStroke(Color.web("#000"));
        pg746.setFill(Color.WHITE);
        pg746.setContent("m29,128.8s-0.2-1.8,1.6-1,175.2-12.2,198.8,20.2c0,0-9.6-25.6-200.4-19.2z");
        getChildren().add(pg746);
        SVGPath pg750 = new SVGPath();
        pg750.setStrokeWidth(0.1);
        pg750.setStroke(Color.web("#000"));
        pg750.setFill(Color.WHITE);
        pg750.setContent("m39,124s-0.2-1.8,1.6-1,124-37.8,147.6-5.4c0,0-13.4-24.6-149.2,6.4z");
        getChildren().add(pg750);
        SVGPath pg754 = new SVGPath();
        pg754.setStrokeWidth(0.1);
        pg754.setStroke(Color.web("#000"));
        pg754.setFill(Color.WHITE);
        pg754.setContent("m-19,146.8s-0.2-1.8,1.6-1,19.6,3,21.6,41.8c0,0-7.2-42-23.2-40.8z");
        getChildren().add(pg754);
        SVGPath pg758 = new SVGPath();
        pg758.setStrokeWidth(0.1);
        pg758.setStroke(Color.web("#000"));
        pg758.setFill(Color.WHITE);
        pg758.setContent("m-27.8,148.4s-0.2-1.8,1.6-1,16-3.8,13.2,35c0,0,1.2-35.2-14.8-34z");
        getChildren().add(pg758);
        SVGPath pg762 = new SVGPath();
        pg762.setStrokeWidth(0.1);
        pg762.setStroke(Color.web("#000"));
        pg762.setFill(Color.WHITE);
        pg762.setContent("m-35.8,148.8s-0.2-1.8,1.6-1,17.2,1.4,4.8,23.8c0,0,9.6-24-6.4-22.8z");
        getChildren().add(pg762);
        SVGPath pg766 = new SVGPath();
        pg766.setStrokeWidth(0.1);
        pg766.setStroke(Color.web("#000"));
        pg766.setFill(Color.WHITE);
        pg766.setContent("m11.526,104.46s-0.444,2,1.105,0.79c16.068-12.628,48.51-71.53,104.2-77.164,0,0-38.312-12.11-105.3,76.374z");
        getChildren().add(pg766);
        SVGPath pg770 = new SVGPath();
        pg770.setStrokeWidth(0.1);
        pg770.setStroke(Color.web("#000"));
        pg770.setFill(Color.WHITE);
        pg770.setContent("m22.726,102.66s-1.363-1.19,0.505-1.81c1.868-0.63,114.31-73.13,153.6-65.164,0,0-27.11-7.51-154.1,66.974z");
        getChildren().add(pg770);
        SVGPath pg774 = new SVGPath();
        pg774.setStrokeWidth(0.1);
        pg774.setStroke(Color.web("#000"));
        pg774.setFill(Color.WHITE);
        pg774.setContent("m1.885,108.77s-0.509,1.6,1.202,0.62c8.975-5.12,12.59-62.331,56.167-63.586,0,0-32.411-14.714-57.369,62.966z");
        getChildren().add(pg774);
        SVGPath pg778 = new SVGPath();
        pg778.setStrokeWidth(0.1);
        pg778.setStroke(Color.web("#000"));
        pg778.setFill(Color.WHITE);
        pg778.setContent("m-18.038,119.79s-1.077,1.29,0.876,1.03c10.246-1.33,31.651-42.598,76.09-37.519,0,0-31.966-14.346-76.966,36.489z");
        getChildren().add(pg778);
        SVGPath pg782 = new SVGPath();
        pg782.setStrokeWidth(0.1);
        pg782.setStroke(Color.web("#000"));
        pg782.setFill(Color.WHITE);
        pg782.setContent("m-6.8,113.67s-0.811,1.47,1.058,0.84c9.799-3.27,22.883-47.885,67.471-51.432,0,0-34.126-7.943-68.529,50.592z");
        getChildren().add(pg782);
        SVGPath pg786 = new SVGPath();
        pg786.setStrokeWidth(0.1);
        pg786.setStroke(Color.web("#000"));
        pg786.setFill(Color.WHITE);
        pg786.setContent("m-25.078,124.91s-0.873,1.04,0.709,0.84c8.299-1.08,25.637-34.51,61.633-30.396,0,0-25.893-11.62-62.342,29.556z");
        getChildren().add(pg786);
        SVGPath pg790 = new SVGPath();
        pg790.setStrokeWidth(0.1);
        pg790.setStroke(Color.web("#000"));
        pg790.setFill(Color.WHITE);
        pg790.setContent("m-32.677,130.82s-1.005,1.05,0.586,0.93c4.168-0.31,34.806-33.39,53.274-17.89,0,0-12.015-18.721-53.86,16.96z");
        getChildren().add(pg790);
        SVGPath pg794 = new SVGPath();
        pg794.setStrokeWidth(0.1);
        pg794.setStroke(Color.web("#000"));
        pg794.setFill(Color.WHITE);
        pg794.setContent("m36.855,98.898s-1.201-1.355,0.731-1.74c1.932-0.384,122.63-58.097,160.59-45.231,0,0-25.94-10.874-161.32,46.971z");
        getChildren().add(pg794);
        SVGPath pg798 = new SVGPath();
        pg798.setStrokeWidth(0.1);
        pg798.setStroke(Color.web("#000"));
        pg798.setFill(Color.WHITE);
        pg798.setContent("m3.4,163.2s-0.2-1.8,1.6-1,17.2,1.4,4.8,23.8c0,0,9.6-24-6.4-22.8z");
        getChildren().add(pg798);
        SVGPath pg802 = new SVGPath();
        pg802.setStrokeWidth(0.1);
        pg802.setStroke(Color.web("#000"));
        pg802.setFill(Color.WHITE);
        pg802.setContent("m13.8,161.6s-0.2-1.8,1.6-1,19.6,3,21.6,41.8c0,0-7.2-42-23.2-40.8z");
        getChildren().add(pg802);
        SVGPath pg806 = new SVGPath();
        pg806.setStrokeWidth(0.1);
        pg806.setStroke(Color.web("#000"));
        pg806.setFill(Color.WHITE);
        pg806.setContent("m20.6,160s-0.2-1.8,1.6-1,26.4,4.2,50,36.6c0,0-35.6-36.8-51.6-35.6z");
        getChildren().add(pg806);
        SVGPath pg810 = new SVGPath();
        pg810.setStrokeWidth(0.1);
        pg810.setStroke(Color.web("#000"));
        pg810.setFill(Color.WHITE);
        pg810.setContent("m28.225,157.97s-0.437-1.76,1.453-1.2c1.89,0.55,22.324-1.35,60.421,32.83,0,0-46.175-34.94-61.874-31.63z");
        getChildren().add(pg810);
        SVGPath pg814 = new SVGPath();
        pg814.setStrokeWidth(0.1);
        pg814.setStroke(Color.web("#000"));
        pg814.setFill(Color.WHITE);
        pg814.setContent("m38.625,153.57s-0.437-1.76,1.453-1.2c1.89,0.55,36.724,5.05,88.422,40.03,0,0-74.176-42.14-89.875-38.83z");
        getChildren().add(pg814);
        SVGPath pg818 = new SVGPath();
        pg818.setStrokeWidth(0.1);
        pg818.setStroke(Color.web("#000"));
        pg818.setFill(Color.WHITE);
        pg818.setContent("m-1.8,142s-0.2-1.8,1.6-1,55.2,3.4,85.6,30.2c0,0-34.901-24.77-87.2-29.2z");
        getChildren().add(pg818);
        SVGPath pg822 = new SVGPath();
        pg822.setStrokeWidth(0.1);
        pg822.setStroke(Color.web("#000"));
        pg822.setFill(Color.WHITE);
        pg822.setContent("m-11.8,146s-0.2-1.8,1.6-1,26.4,4.2,50,36.6c0,0-35.6-36.8-51.6-35.6z");
        getChildren().add(pg822);
        SVGPath pg826 = new SVGPath();
        pg826.setStrokeWidth(0.1);
        pg826.setStroke(Color.web("#000"));
        pg826.setFill(Color.WHITE);
        pg826.setContent("m49.503,148.96s-0.565-1.72,1.361-1.3c1.926,0.41,36.996,2.34,91.116,33.44,0,0-77.663-34.4-92.477-32.14z");
        getChildren().add(pg826);
        SVGPath pg830 = new SVGPath();
        pg830.setStrokeWidth(0.1);
        pg830.setStroke(Color.web("#000"));
        pg830.setFill(Color.WHITE);
        pg830.setContent("m57.903,146.56s-0.565-1.72,1.361-1.3c1.926,0.41,36.996,2.34,91.116,33.44,0,0-77.063-34.8-92.477-32.14z");
        getChildren().add(pg830);
        SVGPath pg834 = new SVGPath();
        pg834.setStrokeWidth(0.1);
        pg834.setStroke(Color.web("#000"));
        pg834.setFill(Color.WHITE);
        pg834.setContent("m67.503,141.56s-0.565-1.72,1.361-1.3c1.926,0.41,44.996,4.74,134.72,39.04,0,0-120.66-40.4-136.08-37.74z");
        getChildren().add(pg834);
        SVGPath pg838 = new SVGPath();
        pg838.setFill(Color.BLACK);
        pg838.setContent("m-43.8,148.4s5.2-0.4,4,1.2-3.6,0.8-3.6,0.8l-0.4-2z");
        getChildren().add(pg838);
        SVGPath pg842 = new SVGPath();
        pg842.setFill(Color.BLACK);
        pg842.setContent("m-13,162.4s5.2-0.4,4,1.2-3.6,0.8-3.6,0.8l-0.4-2z");
        getChildren().add(pg842);
        SVGPath pg846 = new SVGPath();
        pg846.setFill(Color.BLACK);
        pg846.setContent("m-21.8,162s5.2-0.4,4,1.2-3.6,0.8-3.6,0.8l-0.4-2z");
        getChildren().add(pg846);
        SVGPath pg850 = new SVGPath();
        pg850.setFill(Color.BLACK);
        pg850.setContent("m-117.17,150.18s5.05,1.32,3.39,2.44-3.67-0.42-3.67-0.42l0.28-2.02z");
        getChildren().add(pg850);
        SVGPath pg854 = new SVGPath();
        pg854.setFill(Color.BLACK);
        pg854.setContent("m-115.17,140.58s5.05,1.32,3.39,2.44-3.67-0.42-3.67-0.42l0.28-2.02z");
        getChildren().add(pg854);
        SVGPath pg858 = new SVGPath();
        pg858.setFill(Color.BLACK);
        pg858.setContent("m-122.37,136.18s5.05,1.32,3.39,2.44-3.67-0.42-3.67-0.42l0.28-2.02z");
        getChildren().add(pg858);
        SVGPath pg862 = new SVGPath();
        pg862.setFill(Color.web("#CCC"));
        pg862.setContent("m-42.6,211.2-5.6,2c-2,0-13.2,3.6-18.8,13.6,0,0,12.4-9.6,24.4-15.6z");
        getChildren().add(pg862);
        SVGPath pg866 = new SVGPath();
        pg866.setFill(Color.web("#CCC"));
        pg866.setContent("m45.116,303.85c0.141,0.25,0.196,0.67,0.488,0.69,0.658,0.04,1.891,0.34,1.766-0.29-0.848-4.31-1.722-9.25-5.855-11.05-0.639-0.28-2.081,0.13-2.155,1.02-0.127,1.52-0.244,2.87,0.065,4.33,0.3,1.43,2.458,1.43,3.375,0.05,0.936,1.67,1.368,3.52,2.316,5.25z");
        getChildren().add(pg866);
        SVGPath pg870 = new SVGPath();
        pg870.setFill(Color.web("#CCC"));
        pg870.setContent("m34.038,308.58c0.748,1.41,0.621,3.27,2.036,3.84,0.74,0.29,2.59-0.68,2.172-1.76-0.802-2.06-1.19-4.3-2.579-6.11-0.2-0.26,0.04-0.79-0.12-1.12-0.594-1.22-1.739-1.96-3.147-1.63-1.115,2.2,0.033,4.33,1.555,6.04,0.136,0.15-0.03,0.53,0.083,0.74z");
        getChildren().add(pg870);
        SVGPath pg874 = new SVGPath();
        pg874.setFill(Color.web("#CCC"));
        pg874.setContent("m-5.564,303.39c-0.108-0.38-0.146-0.84,0.019-1.16,0.531-1.03,1.324-2.15,0.987-3.18-0.348-1.05-1.464-0.87-2.114-0.3-1.135,0.99-1.184,2.82-1.875,4.18-0.196,0.38-0.145,0.96-0.586,1.35-0.474,0.42-0.914,1.94-0.818,2.51,0.053,0.32-0.13,10.22,0.092,9.96,0.619-0.73,3.669-10.47,3.738-11.36,0.057-0.73,0.789-1.19,0.557-2z");
        getChildren().add(pg874);
        SVGPath pg878 = new SVGPath();
        pg878.setFill(Color.web("#CCC"));
        pg878.setContent("m-31.202,296.6c2.634-2.5,5.424-5.46,4.982-9.17-0.116-0.98-1.891-0.45-2.078,0.39-0.802,3.63-2.841,6.29-5.409,8.68-2.196,2.05-4.058,8.39-4.293,8.9,3.697-5.26,5.954-8,6.798-8.8z");
        getChildren().add(pg878);
        SVGPath pg882 = new SVGPath();
        pg882.setFill(Color.web("#CCC"));
        pg882.setContent("m-44.776,290.64c0.523-0.38,0.221-0.87,0.438-1.2,0.953-1.46,2.254-2.7,2.272-4.44,0.003-0.28-0.375-0.59-0.71-0.36-0.277,0.18-0.619,0.31-0.727,0.44-2.03,2.45-3.43,5.12-4.873,7.93-0.183,0.36-1.327,4.85-1.014,4.96,0.239,0.09,1.959-4.09,2.169-4.21,1.263-0.68,1.275-2.3,2.445-3.12z");
        getChildren().add(pg882);
        SVGPath pg886 = new SVGPath();
        pg886.setFill(Color.web("#CCC"));
        pg886.setContent("m-28.043,310.18c0.444-0.87,2.02-2.07,1.907-2.96-0.118-0.93,0.35-2.37-0.562-1.68-1.257,0.94-4.706,2.29-4.976,8.1-0.026,0.57,2.948-2.12,3.631-3.46z");
        getChildren().add(pg886);
        SVGPath pg890 = new SVGPath();
        pg890.setFill(Color.web("#CCC"));
        pg890.setContent("m-13.6,293c0.4-0.67,1.108-0.19,1.567-0.46,0.648-0.37,1.259-0.93,1.551-1.58,0.97-2.14,2.739-3.96,2.882-6.36-1.491-1.4-2.17,0.64-2.8,1.6-1.323-1.65-2.322,0.23-3.622,0.75-0.07,0.03-0.283-0.32-0.358-0.29-1.177,0.44-1.857,1.52-2.855,2.3-0.171,0.13-0.576-0.05-0.723,0.09-0.652,0.6-1.625,0.93-1.905,1.61-1.11,2.7-4.25,4.8-6.137,12.34,0.381,0.91,4.512-6.64,4.999-7.34,0.836-1.2,0.954,1.66,2.23,1,0.051-0.03,0.237,0.21,0.371,0.34,0.194-0.28,0.412-0.51,0.8-0.4,0-0.4-0.134-0.96,0.067-1.11,1.237-0.98,1.153-2.05,1.933-3.29,0.458,0.79,1.519,0.07,2,0.8z");
        getChildren().add(pg890);
        SVGPath pg894 = new SVGPath();
        pg894.setFill(Color.web("#CCC"));
        pg894.setContent("m46.2,347.4s7.4-20.4,3-31.6c0,0,11.4,21.6,6.8,32.8,0,0-0.4-10.4-4.4-15.4,0,0-4,12.8-5.4,14.2z");
        getChildren().add(pg894);
        SVGPath pg898 = new SVGPath();
        pg898.setFill(Color.web("#CCC"));
        pg898.setContent("m31.4,344.8s5.4-8.8-2.6-27.2c0,0-0.8,20.4-7.6,31.4,0,0,14.2-20.2,10.2-4.2z");
        getChildren().add(pg898);
        SVGPath pg902 = new SVGPath();
        pg902.setFill(Color.web("#CCC"));
        pg902.setContent("m21.4,342.8s-0.2-20,0.2-23c0,0-3.8,16.6-14,26.2,0,0,14.4-12,13.8-3.2z");
        getChildren().add(pg902);
        SVGPath pg906 = new SVGPath();
        pg906.setFill(Color.web("#CCC"));
        pg906.setContent("m11.8,310.8s6,13.6-4,32c0,0,6.4-12.2,1.6-19.2,0,0,2.6-3.4,2.4-12.8z");
        getChildren().add(pg906);
        SVGPath pg910 = new SVGPath();
        pg910.setFill(Color.web("#CCC"));
        pg910.setContent("m-7.4,342.4s-1-15.6,0.8-17.8c0,0,0.2-6.4-0.2-7.4,0,0,4-6.2,4.2,1.2,0,0,1.4,7.8,4.2,12.4,0,0,3.6,5.4,3.4,11.8,0,0-10-30.2-12.4-0.2z");
        getChildren().add(pg910);
        SVGPath pg914 = new SVGPath();
        pg914.setFill(Color.web("#CCC"));
        pg914.setContent("m-11,314.8s-6.6,10.8-8.4,29.8c0,0-1.4-6.2,2.4-20.6,0,0,4.2-15.4,6-9.2z");
        getChildren().add(pg914);
        SVGPath pg918 = new SVGPath();
        pg918.setFill(Color.web("#CCC"));
        pg918.setContent("m-32.8,334.6s5-5.4,6.4-10.4c0,0,3.6-15.8-2.8-7.2,0,0,0.2,8-8,15.4,0,0,4.8-2.4,4.4,2.2z");
        getChildren().add(pg918);
        SVGPath pg922 = new SVGPath();
        pg922.setFill(Color.web("#CCC"));
        pg922.setContent("m-38.6,329.6s3.4-17.4,4.2-18.2c0,0,1.8-3.4-1-0.2,0,0-8.8,19.2-12.8,25.8,0,0,8-9.2,9.6-7.4z");
        getChildren().add(pg922);
        SVGPath pg926 = new SVGPath();
        pg926.setFill(Color.web("#CCC"));
        pg926.setContent("m-44.4,313s11.6-22.4-10.2,3.4c0,0,11-9.8,10.2-3.4z");
        getChildren().add(pg926);
        SVGPath pg930 = new SVGPath();
        pg930.setFill(Color.web("#CCC"));
        pg930.setContent("m-59.8,298.4s4.8-18.8,7.4-18.6l1.6,1.6s-6,9.6-5.4,19.4c0,0-0.6-9.6-3.6-2.4z");
        getChildren().add(pg930);
        SVGPath pg934 = new SVGPath();
        pg934.setFill(Color.web("#CCC"));
        pg934.setContent("m270.5,287s-12-10-14.5-13.5c0,0,13.5,18.5,13.5,25.5,0,0,2.5-7.5,1-12z");
        getChildren().add(pg934);
        SVGPath pg938 = new SVGPath();
        pg938.setFill(Color.web("#CCC"));
        pg938.setContent("m276,265s-21-15-24.5-22.5c0,0,26.5,29.5,26.5,34,0,0,0.5-9-2-11.5z");
        getChildren().add(pg938);
        SVGPath pg942 = new SVGPath();
        pg942.setFill(Color.web("#CCC"));
        pg942.setContent("m293,111s-12-8-13.5-6c0,0,10.5,6.5,13,15,0,0-1.5-9,0.5-9z");
        getChildren().add(pg942);
        SVGPath pg946 = new SVGPath();
        pg946.setFill(Color.web("#CCC"));
        pg946.setContent("m301.5,191.5-17.5-12s19,17,19.5,21l-2-9z");
        getChildren().add(pg946);
        SVGPath pg950 = new SVGPath();
        pg950.setStroke(Color.web("#000"));
        pg950.setContent("m-89.25,169,22,4.75");
        getChildren().add(pg950);
        SVGPath pg954 = new SVGPath();
        pg954.setStroke(Color.web("#000"));
        pg954.setContent("m-39,331s-0.5-3.5-9.5,7");
        getChildren().add(pg954);
        SVGPath pg958 = new SVGPath();
        pg958.setStroke(Color.web("#000"));
        pg958.setContent("m-33.5,336s2-6.5-4.5-2");
        getChildren().add(pg958);
        SVGPath pg962 = new SVGPath();
        pg962.setStroke(Color.web("#000"));
        pg962.setContent("m20.5,344.5s1.5-11-10,2");
        getChildren().add(pg962);
    }

    /**
     * Spits out the Java code needed to draw the tiger. Just feed it the Tiger.svg file,
     * copy the output and replace the code in the constructor with the output.
     *
     * @param args main arguments
     * @throws Exception exceptions from XML parsing
     */
    public static void main(String[] args) throws Exception {
        File file = new File(Tiger.class.getResource("Tiger.svg").toURI());
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        Document dom = factory.newDocumentBuilder().parse(file);

        XPath xpath = XPathFactory.newInstance().newXPath();
        XPathExpression findAll = xpath.compile("/svg/g/g");
        NodeList nodes = (NodeList) findAll.evaluate(dom, XPathConstants.NODESET);

        for (int i=0; i<nodes.getLength(); i++) {
            Element g = (Element) nodes.item(i);
            String id = g.getAttribute("id");
            Double strokeWidth = g.hasAttribute("stroke-width") ? Double.valueOf(g.getAttribute("stroke-width")) : null;
            String stroke = g.hasAttribute("stroke") ? g.getAttribute("stroke") : null;
            String fill = g.hasAttribute("fill") ? g.getAttribute("fill") : null;

            Element SVGPath = (Element) g.getElementsByTagName("SVGPath").item(0);
            String pathString = SVGPath.getAttribute("d");

            String object = "p" + id;
            System.out.println("SVGPath " + object + " = new SVGPath();");
            if (strokeWidth != null) {
                System.out.println(object + ".setStrokeWidth(" + strokeWidth + ");");
            }
            if (stroke != null) {
                System.out.println(object + ".setStroke(Color.web(\"" + stroke + "\"));");
            }
            if (fill != null) {
                if (fill.equals("#000")) fill = "Color.BLACK";
                else if (fill.equals("#FFF")) fill = "Color.WHITE";
                else fill = "Color.web(\"" + fill + "\")";
                System.out.println(object + ".setFill(" + fill + ");");
            }
            System.out.println(object + ".setContent(\"" + pathString + "\");");
            System.out.println("getChildren().add(" + object + ");");

            // m, L, c, z, h, v, s, q, t, a ...?
        }


    }
}
