/*
 * Copyright (c) 2010, 2013, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the LICENSE file that accompanied this code.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */

package javafx.fxml;

import java.io.IOException;
import org.junit.Assert;
import org.junit.Test;

public final class RT_34146Test {
    @Test
    public void testCanCallAbstractMethodFromPrivateClass() throws IOException {
        final FXMLLoader fxmlLoader =
                new FXMLLoader(getClass().getResource("rt_34146.fxml"));

        final ConcreteController concreteController =
                new ConcreteController();
        fxmlLoader.setController(concreteController);
        final Widget widget = fxmlLoader.<Widget>load();
        widget.fire();

        Assert.assertTrue(concreteController.getActionHandlerCalled());
    }

    private static abstract class AbstractController {
        @FXML
        protected abstract void actionHandler();
    }

    private static final class ConcreteController extends AbstractController {
        private boolean actionHandlerCalled;

        @Override
        protected void actionHandler() {
            actionHandlerCalled = true;
        }

        boolean getActionHandlerCalled() {
            return actionHandlerCalled;
        }
    }
}
