/*
 * Copyright (C) 2014 Apple Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL APPLE INC. OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef NSImmediateActionGestureRecognizerSPI_h
#define NSImmediateActionGestureRecognizerSPI_h

#import <AppKit/NSGestureRecognizer.h>

// FIXME: This header should include system headers when possible.

@class NSImmediateActionGestureRecognizer;

@protocol NSImmediateActionGestureRecognizerDelegate <NSGestureRecognizerDelegate>
@optional

- (void)immediateActionRecognizerWillPrepare:(NSImmediateActionGestureRecognizer *)immediateActionRecognizer;
- (void)immediateActionRecognizerWillBeginAnimation:(NSImmediateActionGestureRecognizer *)immediateActionRecognizer;
- (void)immediateActionRecognizerDidUpdateAnimation:(NSImmediateActionGestureRecognizer *)immediateActionRecognizer;
- (void)immediateActionRecognizerDidCancelAnimation:(NSImmediateActionGestureRecognizer *)immediateActionRecognizer;
- (void)immediateActionRecognizerDidCompleteAnimation:(NSImmediateActionGestureRecognizer *)immediateActionRecognizer;

@end

@protocol NSImmediateActionAnimationController <NSObject>
@optional

- (void)recognizerWillBeginAnimation:(NSImmediateActionGestureRecognizer *)recognizer;
- (void)recognizerDidUpdateAnimation:(NSImmediateActionGestureRecognizer *)recognizer;
- (void)recognizerDidCancelAnimation:(NSImmediateActionGestureRecognizer *)recognizer;
- (void)recognizerDidCompleteAnimation:(NSImmediateActionGestureRecognizer *)recognizer;
- (void)recognizerDidDismissAnimation:(NSImmediateActionGestureRecognizer *)recognizer;
@end

@interface NSImmediateActionGestureRecognizer : NSGestureRecognizer

@property (strong) id<NSImmediateActionAnimationController> animationController;
@property (readonly) CGFloat animationProgress;

@end

#endif
